/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: GNOMEInterface.h,v 1.2 2001/03/20 03:43:15 pascal Exp $
 */

#ifndef __GNOMEINTERFACE_H__
#define __GNOMEINTERFACE_H__

#include <gnome.h>

#include "../../renderer/WindowDrawingInterface.h"

/**
 * Implements drawing primitives using GTK / GDK and is used by
 * WMLWindow.
 */
class GNOMEInterface : public WindowDrawingInterface {

public:

  GNOMEInterface(GtkWidget *widget, GdkGC *graphicsContext, GdkColor *fColor, GdkColor *bColor, GdkColor *lColor, GdkColor *alColor, const char *foundry, const char *name, int size);
  virtual ~GNOMEInterface();
  void setPreferences(MobilizerPreferences *mp);
  void startDrawing(int width, int height);
  void stopDrawing();
  int getStringHeight(String *s, int style);
  int getStringWidth(String *s, int style);
  int getPixelsBelowBaseLine(int style);
  void drawText(String *text, int x, int y, int style);
  
  GUIPeer *createInputElement();
  void drawInputElement(WMLGUIElement *wie);
  GUIPeer *createSelectElement(PointerList *optionList);
  void drawSelectElement(WMLGUIElement *wse);
  GUIPeer *createImage(WBMP *wbmp);
  void drawImage(WMLGUIElement *image);
  
  void setFont(const char *foundry, const char *name, int size);
  void setForegroundColor(GdkColor *c);
  void setBackgroundColor(GdkColor *c);
  void setLinkColor(GdkColor *c);
  void setActiveLinkColor(GdkColor *c);

private:
  /** The canvas to draw on. */
  GtkWidget *canvas;
  /** The font to use. */
  GdkFont *currentFont;
  /** The graphics context. */
  GdkGC *gc;
  /** The foreground color. */
  GdkColor *foreColor;
  /** The background color. */
  GdkColor *backColor;
  /** The link color. */
  GdkColor *linkColor;
  /** The active link color. */
  GdkColor *activeLinkColor;
  /**
   * This array contains preloaded fonts for all possible combinations
   * of font styles:
   *  0: nothing
   *  1: bold
   *  2: italic
   *  3: big
   *  4: small
   *  5: bolditalic
   *  6: boldbig
   *  7: boldsmall
   *  8: italicbig
   *  9: italicsmall
   * 10: bolditalicbig
   * 11: bolditalicsmall
   */
  GdkFont *fonts[12]; 
  /** The font foundry (e.g. adobe). */
  char fontFoundry[100];
  /** The font name (e.g. times). */
  char fontName[100];
  /** The base size of the font in pixels (e.g. 18). */
  int fontBaseSize; 
  /** The preferred slant to use: i, o, r. 
      This is a caching variable so that loadFont() doesn't
      have to do the expensive fallback every time. */
  char preferredSlant[2];
  /** The preferred weight to use: bold, medium.
      This is a caching variable so that loadFont() doesn't
      have to do the expensive fallback every time. */
  char preferredWeight[100];
  /** Whether or not to use the "fixed" font.
      Another caching variable. */
  bool useFixed;

  GdkFont *loadFont(int style);
  void createFontStyles();
  void destroyFontStyles();
  void setFontStyle(int style);
  void createLFD(char *str, int len, int style, const char *slant, const char *weight);


};


#endif

