/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: GdkPixmapPeer.cpp,v 1.1 2001/03/11 08:43:40 pascal Exp $ */

#include <stdio.h>
#include <gnome.h>

#include "../../lib/String.h"
#include "../../renderer/WBMP.h"
#include "../../renderer/GUIPeer.h"
#include "GdkPixmapPeer.h"

extern GdkColormap *colorMap;

/**
 * Creates a GdkPixmap from the given WBMP.
 * @param canvas the canvas to attach to.
 * @param image the image to create.
 */
GdkPixmapPeer::GdkPixmapPeer(GtkWidget *canvas, GdkColor *fgColor, GdkColor *bgColor, WBMP *image) {
  unsigned char *wbmp;
  int i;

  width = image->getWidth();
  height = image->getHeight();
  
  //  gdk_colormap_alloc_color(colorMap, fgColor, 0, 1);
  //  gdk_colormap_alloc_color(colorMap, bgColor, 0, 1);
  
  wbmp = (unsigned char *) image->getImageData();
  
  int widthInBytes;
  
  if(width % 8)
    widthInBytes = width / 8 + 1;
  else
    widthInBytes = width / 8;
    
  unsigned char bits[widthInBytes * 8 * height];
    
  for(i = 0; i < widthInBytes * 8 * height; i++) {
   
    bool bit;
    
    switch(i%8) {
    
    case 0:
      bit = wbmp[i/8] & 128;
      bits[i/8] = 1 * (int) bit; /* notice: '=' in stead of '|=' */
      break;                     /* setting all bits to 0 first  */
    case 1:
      bit = wbmp[i/8] & 64;
      bits[i/8] |= 2 * (int) bit;
      break;
    case 2:
      bit = wbmp[i/8] & 32;
      bits[i/8] |= 4 * (int) bit;
      break;
    case 3:
      bit = wbmp[i/8] & 16;
      bits[i/8] |= 8 * (int) bit;
      break;
    case 4:
      bit = wbmp[i/8] & 8;
      bits[i/8] |= 16 * (int) bit;
      break;
    case 5:
      bit = wbmp[i/8] & 4;
      bits[i/8] |= 32 * (int) bit;
      break;
    case 6:
      bit = wbmp[i/8] & 2;
      bits[i/8] |= 64 * (int) bit;
      break;
    case 7:
      bit = wbmp[i/8] & 1;
      bits[i/8] |= 128 * (int) bit;
      break;
      
    }
             
  }
  
  pointer = gdk_pixmap_create_from_data(gtk_widget_get_parent_window(canvas), 
                                        (gchar *) bits, 
                                        width, 
                                        height, 
                                        -1, 
                                        bgColor, 
                                        fgColor);

}

GdkPixmapPeer::~GdkPixmapPeer() {
  
  gdk_pixmap_unref((GdkPixmap *) pointer);

}
