/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: GtkCallbacks.h,v 1.4 2001/03/22 04:56:47 pascal Exp $ 
 */

#ifndef __GTK_CALLBACKS_H__
#define __GTK_CALLBACKS_H__

#include <gnome.h>

#include "../../renderer/UpdateWindowList.h"
#include "../../renderer/WMLWindow.h"
#include "FontSupport.h"
#include "Mobilizer.h"

/* Utility functions. */
void gslist_free_family(void *family, void *dummy);
gint gslist_compare_families(gconstpointer a, gconstpointer b);

void process_updates(BrowserInterface *bi, UpdateWindowList *uwl);
void contentarea_clear(BrowserInterface *bi);
void contentarea_draw(BrowserInterface *bi);
void contentarea_refresh(BrowserInterface *bi);

void file_selection_create(BrowserInterface *bi);

/* Callbacks. */
/* C linkage required for Gtk callbacks. */
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

void file_selection_complete(GtkFileSelection *selector, BrowserInterface *bi);

gboolean on_contentarea_size_allocate(GtkWidget *w, GtkAllocation *allocation, BrowserInterface *bi);
gboolean on_contentarea_draw(GtkWidget *widget, GdkRectangle *area, BrowserInterface *bi);
gboolean on_contentarea_expose_event(GtkWidget *widget, GdkEventExpose *event, BrowserInterface *bi);
gboolean on_contentarea_button_press_event(GtkWidget *w, GdkEvent *ev, BrowserInterface *bi);
gboolean on_contentarea_key_press_event(GtkWidget *w, GdkEvent *ev, BrowserInterface *bi);
void on_contentarea_motion_notify_event(GtkWidget *w, GdkEvent *ev, BrowserInterface *bi);

void on_location_activate(GtkWidget *w, BrowserInterface *bi);

void on_about_activate(GtkWidget *w, BrowserInterface *bi);

void on_open_clicked(GtkWidget *w, BrowserInterface *bi);
void on_home_clicked(GtkWidget *w, BrowserInterface *bi);
void on_reload_clicked(GtkWidget *w, BrowserInterface *bi);
void on_back_clicked(GtkWidget *w, BrowserInterface *bi);
void on_forward_clicked(GtkWidget *w, BrowserInterface *bi);

void on_fontsize_changed();
void on_fontsize_insert_text(GtkEditable *w, gchar *new_text, gint new_text_length, gint *position, gpointer user_data);
void on_fontfamilies_menu_deactivate(GtkMenuItem *menuitem);

void on_preferences_activate(GtkWidget *w, BrowserInterface *bi);
void on_preferences_apply(GnomePropertyBox *box, int page, BrowserInterface *bi);
void on_preferences_close();

void on_background_color_set(GnomeColorPicker *colorpicker, guint arg1, guint arg2, guint arg3, guint arg4, gpointer user_data);
void on_text_color_set(GnomeColorPicker *colorpicker, guint arg1, guint arg2, guint arg3, guint arg4, gpointer user_data);
void on_link_color_set(GnomeColorPicker *colorpicker, guint r, guint g, guint b, guint a, gpointer user_data);
void on_activelink_color_set(GnomeColorPicker *colorpicker, guint r, guint g, guint b, guint a, gpointer user_data);

void on_exit1_activate();

#ifdef __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

#endif /* __GTK_CALLBACKS_H__ */

