/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: Mobilizer.cpp,v 1.6 2001/03/25 21:57:43 pascal Exp $ 
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <gnome.h>
#include <ctype.h>

/* This hack gets around a problem in some older versions of glade.h,
   which do not play nicely with C++. */
#ifdef __cplusplus
#undef __cplusplus
#define __plusplus
extern "C" {
#endif
#include <glade/glade.h>
#ifdef __plusplus
#define __cplusplus
} /* extern "C" */
#endif

#include "../../lib/String.h"
#include "../../lib/ContentSink.h"
#include "../../renderer/WMLWindow.h"
#include "../../renderer/WindowDrawingInterface.h"

#include "FontSupport.h"
#include "GNOMEInterface.h"
#include "Mobilizer.h"
#include "GtkCallbacks.h"

/**
 * Structure containing all of Mobilizer's global data.
 */
MobilizerGlobals mg;

/**
 * Converts a filename to a URI.
 * @param filename the filename (absolute or relative) to convert.
 * @return a new URI object.
 */
URI *filenameToURI(char *filename) {
  char path_buf[1024];
  char *cwd = getcwd(path_buf, 1024);

  String s("file://");

  if(filename[0] != '/') {

    s.append(cwd);
    s.append("/");

  }

  s.append(filename);

  return new URI(s.getPtr());
}

/**
 * Loads a new URI into the WMLWindow.
 * @param bi the browser interface.
 * @param u the URI to load into.
 */
void loadSource(BrowserInterface *bi, URI *u) {

  bi->ww->setSource(u);
  contentarea_refresh(bi);
  GtkWidget *gtk_entry = gnome_entry_gtk_entry(GNOME_ENTRY(bi->location));
  gtk_entry_set_text(GTK_ENTRY(gtk_entry), u->toString());

  
}

/**
 * Sets up the given browser interface.
 * @param bi the browser interface to set up.
 */
void setupBrowserInterface(BrowserInterface *bi) {
  int fontCount;
  char **fontList;
  char *lfd;
  GladeXML *xml;

  /* Get the default font size or use 18 if there is no default. */
  mg.currentFont.size = 
    gnome_config_private_get_int_with_default("/mobilizer/defaultFont/size=18", NULL);
  lfd = gnome_config_private_get_string("/mobilizer/defaultFont/lfd");

  /* Check to see if a font preference was previously stored. */
  if(lfd) {

    /* Yes, use it. */
    mg.currentFont.family = createFontFamily(lfd);

  } else {

    /* If no font preference was stored, we get a list of fonts from
       the X server and use the first font we find. */
    getFontList(&fontList, &fontCount);
    mg.currentFont.family = createFontFamily(fontList[0]);
    freeFontList(fontList);

  }

  /* Load or setup default colors. */
  bi->colorMap = gdk_colormap_get_system();

  mg.currentColors.background.red = gnome_config_private_get_int_with_default("/mobilizer/backgroundColor/red=65535", NULL);
  mg.currentColors.background.green = gnome_config_private_get_int_with_default("/mobilizer/backgroundColor/green=65535", NULL);
  mg.currentColors.background.blue = gnome_config_private_get_int_with_default("/mobilizer/backgroundColor/blue=65535", NULL);
  mg.currentColors.background.pixel = (gulong) (mg.currentColors.background.red * 65536 + 
                                                mg.currentColors.background.green * 256 + 
                                                mg.currentColors.background.blue);
  gdk_colormap_alloc_color(bi->colorMap, 
                           &(mg.currentColors.background), 
                           0, 1);

  mg.currentColors.foreground.red = gnome_config_private_get_int_with_default("/mobilizer/foregroundColor/red=0", NULL);
  mg.currentColors.foreground.green = gnome_config_private_get_int_with_default("/mobilizer/foregroundColor/green=0", NULL);
  mg.currentColors.foreground.blue = gnome_config_private_get_int_with_default("/mobilizer/foregroundColor/blue=0", NULL);
  mg.currentColors.foreground.pixel = (gulong) (mg.currentColors.foreground.red * 65536 + 
                                                mg.currentColors.foreground.green * 256 + 
                                                mg.currentColors.foreground.blue);
  gdk_colormap_alloc_color(bi->colorMap, 
                           &(mg.currentColors.foreground), 
                           0, 1);

  mg.currentColors.link.red = gnome_config_private_get_int_with_default("/mobilizer/linkColor/red=0", NULL);
  mg.currentColors.link.green = gnome_config_private_get_int_with_default("/mobilizer/linkColor/green=0", NULL);
  mg.currentColors.link.blue = gnome_config_private_get_int_with_default("/mobilizer/linkColor/blue=65535", NULL);
  mg.currentColors.link.pixel = (gulong) (mg.currentColors.link.red * 65536 + 
                                          mg.currentColors.link.green * 256 + 
                                          mg.currentColors.link.blue);
  gdk_colormap_alloc_color(bi->colorMap, 
                           &(mg.currentColors.link), 
                           0, 1);

  mg.currentColors.activeLink.red = gnome_config_private_get_int_with_default("/mobilizer/activeLinkColor/red=0", NULL);
  mg.currentColors.activeLink.green = gnome_config_private_get_int_with_default("/mobilizer/activeLinkColor/green=0", NULL);
  mg.currentColors.activeLink.blue = gnome_config_private_get_int_with_default("/mobilizer/activeLinkColor/blue=65535", NULL);
  mg.currentColors.activeLink.pixel = (gulong) (mg.currentColors.activeLink.red * 65536 + 
                                                mg.currentColors.activeLink.green * 256 + 
                                                mg.currentColors.activeLink.blue);
  gdk_colormap_alloc_color(bi->colorMap, 
                           &(mg.currentColors.activeLink), 
                           0, 1);

  bi->horizontalScrollBarPolicy = GTK_POLICY_NEVER;
  bi->verticalScrollBarPolicy = GTK_POLICY_NEVER;

  /* Load the main window. */
  xml = glade_xml_new(mg.gladeInterface, "app1");

  if(!xml) {

    /* In case we can't load the interface, bail */
    g_warning("We could not load the interface!");
    exit(1);

  }

  /* Get pointers to interface elements. */
  bi->scrolled_window = glade_xml_get_widget(xml, "scrolled_window1");
  bi->home = glade_xml_get_widget(xml, "home");
  bi->reload = glade_xml_get_widget(xml, "reload");
  bi->open = glade_xml_get_widget(xml, "open");
  bi->contentarea = glade_xml_get_widget(xml, "contentarea");
  bi->location = glade_xml_get_widget(xml, "location");
  bi->statusbar = glade_xml_get_widget(xml, "statusbar");
  bi->historyInterface.back = glade_xml_get_widget(xml, "back_button");
  bi->historyInterface.forward = glade_xml_get_widget(xml, "forward_button");

  /* Get a graphics context for drawing. */
  bi->gc = gdk_gc_new(GTK_LAYOUT(bi->contentarea)->bin_window);

  if(bi->gc == NULL) {
    
    g_warning("Could not get a graphics context!");
    exit(1);

  }

  /* Create mouse pointers. */
  bi->mouseCursors.hand = gdk_cursor_new(GDK_HAND2);
  bi->mouseCursors.standard = gdk_cursor_new(GDK_TOP_LEFT_ARROW);

  /* Start off with dummy values for the dimensions of our content
     area. */
  bi->width = 100;
  bi->height = 100;

  /* Connect signals. */
  GtkWidget *w;

  /* There is an event box behind the GtkLayout called contentarea,
     otherwise we don't get events on RedHat 6.2 systems. */
  w = glade_xml_get_widget(xml, "eventbox1");

  gtk_widget_add_events(w, GDK_POINTER_MOTION_MASK);

  gtk_signal_connect(GTK_OBJECT(w), "button-press-event",
		     GTK_SIGNAL_FUNC(on_contentarea_button_press_event), bi);

  gtk_signal_connect(GTK_OBJECT(w), "key-press-event",
		     GTK_SIGNAL_FUNC(on_contentarea_key_press_event), bi);

  gtk_signal_connect(GTK_OBJECT(w), "motion-notify-event",
		     GTK_SIGNAL_FUNC(on_contentarea_motion_notify_event), bi);

  /* Now setup the rest of the signals. */
  w = glade_xml_get_widget(xml, "app1");

  gtk_signal_connect(GTK_OBJECT(w), "destroy",
		     GTK_SIGNAL_FUNC(gtk_main_quit), bi);

  w = glade_xml_get_widget(xml, "about");

  gtk_signal_connect(GTK_OBJECT(w), "activate",
		     GTK_SIGNAL_FUNC(on_about_activate), bi);

  w = glade_xml_get_widget(xml, "preferences1");

  gtk_signal_connect(GTK_OBJECT(w), "activate",
		     GTK_SIGNAL_FUNC(on_preferences_activate), bi);

  gtk_signal_connect(GTK_OBJECT(bi->open), "clicked",
		     GTK_SIGNAL_FUNC(on_open_clicked), bi);

  gtk_signal_connect(GTK_OBJECT(bi->reload), "clicked",
		     GTK_SIGNAL_FUNC(on_reload_clicked), bi);

  gtk_signal_connect(GTK_OBJECT(bi->home), "clicked",
		     GTK_SIGNAL_FUNC(on_home_clicked), bi);

  gtk_signal_connect(GTK_OBJECT(bi->historyInterface.back), "clicked",
		     GTK_SIGNAL_FUNC(on_back_clicked), bi);

  gtk_signal_connect(GTK_OBJECT(bi->historyInterface.forward), "clicked",
		     GTK_SIGNAL_FUNC(on_forward_clicked), bi);

  gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(bi->location))), "activate", GTK_SIGNAL_FUNC(on_location_activate), bi);

  gtk_signal_connect(GTK_OBJECT(bi->contentarea), "draw",
		     GTK_SIGNAL_FUNC(on_contentarea_draw), bi);

  gtk_signal_connect(GTK_OBJECT(bi->contentarea), "size-allocate",
		     GTK_SIGNAL_FUNC(on_contentarea_size_allocate), bi);

  gtk_signal_connect(GTK_OBJECT(bi->contentarea), "expose-event",
		     GTK_SIGNAL_FUNC(on_contentarea_expose_event), bi);

  gtk_signal_connect(GTK_OBJECT(bi->contentarea), "size-allocate",
		     GTK_SIGNAL_FUNC (on_contentarea_size_allocate), bi);

  /* Setup scrollbars. */
  bi->horizontalAdjustment = 
    GTK_ADJUSTMENT(gtk_adjustment_new(0,   /* value */
				      0,   /* lower */
				      0,   /* upper */
				      10,   /* step_increment */
				      0,   /* page_increment */
				      0)); /* page_size */
  
  bi->verticalAdjustment = 
    GTK_ADJUSTMENT(gtk_adjustment_new(0,   /* value */
				      0,   /* lower */
				      0,   /* upper */
				      10,   /* step_increment */
				      0,   /* page_increment */
				      0)); /* page_size */

  gtk_scrolled_window_set_vadjustment(GTK_SCROLLED_WINDOW(bi->scrolled_window),
                                      bi->verticalAdjustment);

  gtk_scrolled_window_set_hadjustment(GTK_SCROLLED_WINDOW(bi->scrolled_window),
                                      bi->horizontalAdjustment);

  /* Hide history buttons. */
  gtk_widget_set_sensitive(bi->historyInterface.back, FALSE);
  gtk_widget_set_sensitive(bi->historyInterface.forward, FALSE);

  /* Set the background color of the content window. */
  gdk_window_set_background(GTK_LAYOUT(bi->contentarea)->bin_window, 
                            &(mg.currentColors.background));

  /* Create the WMLWindow and its drawing interface. */
  bi->wdi = new GNOMEInterface(bi->contentarea,
                               bi->gc,
                               &(mg.currentColors.foreground),
                               &(mg.currentColors.background),
                               &(mg.currentColors.link),
                               &(mg.currentColors.activeLink),
                               mg.currentFont.family->foundry,
                               mg.currentFont.family->name,
                               mg.currentFont.size);
  bi->ww = new WMLWindow(bi->wdi);

  /* We don't need the GladeXML object any more, so unref it
     to save some memory. */
  gtk_object_unref(GTK_OBJECT(xml));

}

/**
 * Sets up globals, initializes Gnome, etc.
 * @param argc the number of arguments.
 * @param argv the argument vector.
 */
void setupGlobals(int argc, char *argv[]) {
  char *base_dir, *self_name;

  base_dir = (char *) malloc(strlen(argv[0]) + 1);
  mg.gladeInterface = (char *) malloc(strlen(argv[0]) + strlen("mobilizer.glade") + 2);

  if(base_dir == NULL || mg.gladeInterface == NULL) {

    g_warning("Not enough memory.");
    exit(1);

  }

  strcpy(base_dir, argv[0]);

  self_name = strrchr(base_dir, '/');
  if(self_name == NULL) {

    g_warning("Couldn't get base directory.\n");
    exit(1);

  }

  *self_name = '\0';
  self_name++;

  strcpy(mg.gladeInterface, base_dir);
  strcat(mg.gladeInterface, "/");
  strcat(mg.gladeInterface, "mobilizer.glade");

  /* Initialize gnome. */
  gnome_init("Mobilizer", VERSION, argc, argv);
  glade_gnome_init();

}

/**
 * Entry point.
 */
int main(int argc, char *argv[]) {

  /* Initialize ourselves. */
  setupGlobals(argc, argv);
  setupBrowserInterface(&(mg.browserInterface));

  /* Process some events to realize widgets etc. */
  while(gtk_events_pending())
    gtk_main_iteration_do(0);

  /* If an argument was given on the commandline, assume it's a URL to
     load. */
  if(argc > 1)    
    loadSource(&(mg.browserInterface), new URI(argv[1]));
  else
    loadSource(&(mg.browserInterface), new URI("about:blank"));

  /* Run the main loop */
  gtk_main();
  
  return 0;

}

