/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: TextFunctions.cpp,v 1.1 2001/03/14 13:35:48 wouter Exp $ 
 */
 
#include <windows.h>
#include <string.h>
#include <math.h>
#include "TextFunctions.h"

HFONT createFont (HDC hdc, char * szFaceName, int iDeciPtHeight,
                    int iDeciPtWidth, int iAttributes, BOOL fLogRes) {

  FLOAT      cxDpi, cyDpi ;
  HFONT      hFont ;
  LOGFONT    lf ;
  POINT      pt ;
  TEXTMETRIC tm ;

  SaveDC (hdc) ;

  SetGraphicsMode (hdc, GM_ADVANCED) ;
  ModifyWorldTransform (hdc, NULL, MWT_IDENTITY) ;
  SetViewportOrgEx (hdc, 0, 0, NULL) ;
  SetWindowOrgEx   (hdc, 0, 0, NULL) ;

  if (fLogRes) {
    cxDpi = (FLOAT) GetDeviceCaps (hdc, LOGPIXELSX) ;
    cyDpi = (FLOAT) GetDeviceCaps (hdc, LOGPIXELSY) ;
  } else {

    cxDpi = (FLOAT) (25.4 * GetDeviceCaps (hdc, HORZRES) /
                            GetDeviceCaps (hdc, HORZSIZE)) ;

    cyDpi = (FLOAT) (25.4 * GetDeviceCaps (hdc, VERTRES) /
                            GetDeviceCaps (hdc, VERTSIZE)) ;
  }

  pt.x = (int) (iDeciPtWidth  * cxDpi / 72) ;
  pt.y = (int) (iDeciPtHeight * cyDpi / 72) ;

  DPtoLP (hdc, &pt, 1) ;

  lf.lfHeight         = - (int) (fabs (pt.y) / 10.0 + 0.5) ;
  lf.lfWidth          = 0 ;
  lf.lfEscapement     = 0 ;
  lf.lfOrientation    = 0 ;
  lf.lfWeight         = iAttributes & ATTR_BOLD      ? 700 : 0 ;
  lf.lfItalic         = iAttributes & ATTR_ITALIC    ?   1 : 0 ;
  lf.lfUnderline      = iAttributes & ATTR_UNDERLINE ?   1 : 0 ;
  lf.lfStrikeOut      = iAttributes & ATTR_STRIKEOUT ?   1 : 0 ;
  lf.lfCharSet        = 0 ;
  lf.lfOutPrecision   = 0 ;
  lf.lfClipPrecision  = 0 ;
  lf.lfQuality        = 0 ;
  lf.lfPitchAndFamily = 0 ;

  strcpy (lf.lfFaceName, szFaceName) ;

  hFont = CreateFontIndirect (&lf) ;

  if (iDeciPtWidth != 0) {

    hFont = (HFONT) SelectObject (hdc, hFont) ;
    GetTextMetrics (hdc, &tm) ;
    DeleteObject (SelectObject (hdc, hFont)) ;
    lf.lfWidth = (int) (tm.tmAveCharWidth *
                        fabs (pt.x) / fabs (pt.y) + 0.5) ;
    hFont = CreateFontIndirect (&lf) ;

  }
  RestoreDC (hdc, -1) ;
  return hFont ;

}
