/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: WindowsSelectElementPeer.cpp,v 1.2 2001/03/14 13:35:48 wouter Exp $ 
 */
 
#include "WindowsSelectElementPeer.h"
#include "../../lib/PointerList.h"
#include "../../lib/String.h"

/** 
 * Constructs a new WindowsSelectElementPeer object.
 * @param windowHandle the handle to the parent window
 * @param instance     the handle to the instance of the application
 * @param font         the font that must be used for the select element
 * @param optionList   a pointerlist of String object containing the 
 * options
 */
WindowsSelectElementPeer::WindowsSelectElementPeer(HWND windowHandle,
                                                 HINSTANCE instance,
                                                 HFONT font,
                                                 PointerList *optionList) {
  pointer = CreateWindow(
        "listbox",
        NULL,
        WS_CHILD|WS_VISIBLE|WS_VSCROLL|WS_DLGFRAME,
        1,1,2,2,
        windowHandle,
        (HMENU)1,
        instance,
        NULL);

  height = selectElementHeight;
  width = selectElementWidth;

  SendMessage((HWND)pointer,WM_SETFONT,(WPARAM)font,false);

  String* s = (String *)optionList->getFirstAdded();

  while(s) {

      SendMessage((HWND)pointer,LB_ADDSTRING,0,(LPARAM)s->getPtr());
      delete optionList->removeFirstAdded();
      s = (String *)optionList->getFirstAdded();

  }

}

/** 
 * Destroys the object. Destroys the select window.
 */
WindowsSelectElementPeer::~WindowsSelectElementPeer() {

  DestroyWindow((HWND) pointer);

}
