/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: RenderWindow.cpp,v 1.9 2001/03/23 15:46:34 wouter Exp $ 
 */

#include <windows.h>
#include "resource.h"
#include "../TextFunctions.h"
#include "../../../lib/String.h"
#include "../../../lib/URI.h"
#include "../../../renderer/WMLWindow.h"
#include "../../../renderer/UpdateWindowList.h"
#include "../../../lib/MobilizerPreferences.h"
#include "../WindowsInterface.h"
#include <winuser.h>
#include <commctrl.h>
#include <stdio.h>

#define WindowMinWidth 300
#define WindowMinHeight 380
#define BalkQuotient 0.35
#define WindowOffset 10
#define TextOutputWindowQuotient 0.30
#define ButtonWidth 80
#define ButtonHeight 20
#define ContentHeightQuotient 0.8
#define ContentWidthQuotient 0.8

enum windowInterface {
  outputWindow,
  locationWindow,
  buttonLeft,
  buttonMiddle,
  buttonRight,
  textLeft,
  textMiddle,
  textRight,
  contentWindow};
HWND MainWindow;
double contentWQ,contentHQ;
WMLWindow* ww;
MobilizerPreferences* mp;

// Foward declarations of functions included in this code module:
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	ContentProc(HWND, UINT, WPARAM, LPARAM);
long PopFileLength (FILE *file);
void updateWindow(UpdateWindowList* uwl);

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow) {

  InitCommonControls();
  WNDCLASSEX wcex;
	wcex.cbSize         = sizeof(WNDCLASSEX); 
	wcex.style			    = 0;
	wcex.lpfnWndProc	  = (WNDPROC)WndProc;
	wcex.cbClsExtra		  = 0;
	wcex.cbWndExtra		  = 0;
	wcex.hInstance		  = hInstance;
	wcex.hIcon			    = NULL;
	wcex.hCursor		    = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_MENU+1);
	wcex.lpszMenuName	  = NULL;
	wcex.lpszClassName	= "Renderer";
	wcex.hIconSm		    = NULL;

	RegisterClassEx(&wcex);

	wcex.lpfnWndProc	  = (WNDPROC)ContentProc;
  wcex.hbrBackground	= 0;
	wcex.lpszClassName	= "ContentWindow";

	RegisterClassEx(&wcex);

  MainWindow = CreateWindow("Renderer", "Mobilizer", WS_OVERLAPPEDWINDOW,
                        0, 0, 350, 430, NULL,
                        LoadMenu(hInstance,MAKEINTRESOURCE(RENDER_MENU)), hInstance, NULL);

  if(!MainWindow) return FALSE;

  ShowWindow(MainWindow, SW_SHOW);
  UpdateWindow(MainWindow);

 	MSG msg;
  
	while(GetMessage(&msg, NULL, 0, 0)) {

    TranslateMessage(&msg);
    DispatchMessage(&msg);

	}
	return msg.wParam;

}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {

  static HFONT hFont;
	HDC hdc;
  RECT rt;
  static HWND textOutput,locationText,varText,okText,backText,content;
  static WindowsInterface* wi;
  int cx, cy;

	switch (message) {

    case WM_CREATE:     

      hdc = GetDC(hWnd);
      hFont = createFont(hdc,"Arial",90,0,0,TRUE);
      ReleaseDC(hWnd,hdc);

      textOutput = CreateWindowEx(WS_EX_CLIENTEDGE,"edit",NULL,WS_CHILD|WS_VISIBLE|ES_LEFT|ES_MULTILINE|ES_READONLY,
        0,0,0,0,
        hWnd,
        (HMENU)outputWindow,
        ((LPCREATESTRUCT)lParam)->hInstance,NULL);

      SendMessage(textOutput,WM_SETFONT,(WPARAM)hFont,false);

      locationText = CreateWindowEx(WS_EX_CLIENTEDGE,"edit",NULL,WS_CHILD|WS_VISIBLE|ES_LEFT|ES_AUTOHSCROLL|ES_MULTILINE,
        0,0,0,0,
        hWnd,
        (HMENU)locationWindow,
        ((LPCREATESTRUCT)lParam)->hInstance,NULL);

      SendMessage(locationText,WM_SETFONT,(WPARAM)hFont,false);
      SendMessage(locationText,WM_SETTEXT,0,(LPARAM)"http://www.catchy.net/wap/index.wml");

      varText = CreateWindow("combobox",NULL,WS_CHILD|WS_VISIBLE|CBS_DROPDOWNLIST|CBS_SORT|WS_VSCROLL,
        1,1,1,1,
        hWnd,
        (HMENU)textLeft,
        ((LPCREATESTRUCT)lParam)->hInstance,NULL);

      SendMessage(varText,WM_SETFONT,(WPARAM)hFont,false);

      okText = CreateWindow("button","Back",WS_CHILD|WS_VISIBLE|WS_DISABLED,
        1,1,1,1,
        hWnd,
        (HMENU)textMiddle,
        ((LPCREATESTRUCT)lParam)->hInstance,NULL);

      SendMessage(okText,WM_SETFONT,(WPARAM)hFont,false);

      backText = CreateWindow("button","Forward",WS_CHILD|WS_VISIBLE|WS_DISABLED,
        1,1,1,1,
        hWnd,
        (HMENU)textRight,
        ((LPCREATESTRUCT)lParam)->hInstance,NULL);

      SendMessage(backText,WM_SETFONT,(WPARAM)hFont,false);

      contentWQ = ContentWidthQuotient;
      contentHQ = ContentHeightQuotient;

      GetClientRect(hWnd,&rt);

      content = CreateWindow("ContentWindow", NULL, WS_CHILDWINDOW|WS_VISIBLE|WS_SIZEBOX,
        0,0,(int) (contentWQ * (double)rt.right), (int)(contentHQ * (1 - BalkQuotient) * (double)rt.bottom),
        hWnd,
        (HMENU)contentWindow,
        ((LPCREATESTRUCT)lParam)->hInstance, NULL);

      mp = new MobilizerPreferences();
      wi = new WindowsInterface(content,((LPCREATESTRUCT)lParam)->hInstance,hFont);
      wi->setPreferences(mp);
      ww = new WMLWindow(wi);

      return 0;

    case WM_SIZE:
      cx = LOWORD(lParam);
      cy = HIWORD(lParam);
      
      MoveWindow(textOutput,
        WindowOffset,
        (int)((double) cy * (1 - TextOutputWindowQuotient * BalkQuotient)),
        cx - 2 * WindowOffset,
        cy - (int)((double) cy * (1 - TextOutputWindowQuotient * BalkQuotient)) - WindowOffset,
        TRUE);

      MoveWindow(locationText,
        WindowOffset,
        (int)((double) cy * (1 - 2* TextOutputWindowQuotient * BalkQuotient)),
        cx - 2 * WindowOffset,
        ButtonHeight,
        TRUE);

      MoveWindow(varText,
        WindowOffset,
        (int)((double) cy * (1 - BalkQuotient)) + WindowOffset,
        ButtonWidth,
        ButtonHeight*4,
        TRUE);

      MoveWindow(okText,
        (cx / 2) - (ButtonWidth / 2),
        (int)((double) cy * (1 - BalkQuotient)) + WindowOffset,
        ButtonWidth,
        ButtonHeight,
        TRUE);

      MoveWindow(backText,
        cx - ButtonWidth - WindowOffset,
        (int)((double) cy * (1 - BalkQuotient)) + WindowOffset,
        ButtonWidth,
        ButtonHeight,
        TRUE);

      GetWindowRect(content,&rt);

			MoveWindow(content,0,0,10,10,TRUE); /* the WM_SIZE message must be sent */

      MoveWindow(content,
        cx / 2 - (int)((double)cx * contentWQ / 2),
        (int)((double)cy * (1 - BalkQuotient) / 2) - (int)((double)cy * (1 - BalkQuotient) * contentHQ / 2),
        (int)((double)cx * contentWQ),
        (int)((double)cy * (1-BalkQuotient) * contentHQ),
        TRUE);

      return 0;
     
    case WM_SIZING:
      if(wParam == WMSZ_RIGHT || wParam == WMSZ_TOPRIGHT || wParam == WMSZ_BOTTOMRIGHT) {
        if(((RECT*)lParam)->right - ((RECT*)lParam)->left < WindowMinWidth)
          ((RECT*)lParam)->right = ((RECT*)lParam)->left + WindowMinWidth;
      }
      else {
        if(((RECT*)lParam)->right - ((RECT*)lParam)->left < WindowMinWidth)
          ((RECT*)lParam)->left = ((RECT*)lParam)->right - WindowMinWidth;
      }

      if(wParam == WMSZ_BOTTOMLEFT || wParam == WMSZ_BOTTOM || wParam == WMSZ_BOTTOMRIGHT) {
        if(((RECT*)lParam)->bottom - ((RECT*)lParam)->top < WindowMinHeight)
          ((RECT*)lParam)->bottom = ((RECT*)lParam)->top + WindowMinHeight;
      }
      else {
        if(((RECT*)lParam)->bottom - ((RECT*)lParam)->top < WindowMinHeight)
          ((RECT*)lParam)->top = ((RECT*)lParam)->bottom - WindowMinHeight;
      }

      return true;

    case WM_COMMAND:			
    
      if(LOWORD(wParam) == FILE_QUIT)     /* close the window*/
			  DestroyWindow(hWnd);

      if(LOWORD(wParam) == ID_HELP_ABOUT) {
			  
        MessageBox(hWnd,
"(c) 2001 catchy.net\n\n"
"Authors: \n"
"Pascal Haakmat <a.haakmat@chello.nl>,\n"
"Wouter van der Kamp <wouter@catchy.net>\n\n"
"Scintillate, scintillate, asteroid minikin",
"About Mobilizer",MB_ICONINFORMATION|MB_OK);

      }

      if(LOWORD(wParam) == textMiddle) {

        ww->historyGo(-1);
        updateWindow(ww->getUpdates());
        InvalidateRect(hWnd,NULL,false);
        UpdateWindow(hWnd);
        SetFocus(hWnd);

      }

      if(LOWORD(wParam) == textRight) {

        ww->historyGo(1);
        updateWindow(ww->getUpdates());
        InvalidateRect(hWnd,NULL,false);
        UpdateWindow(hWnd);
        SetFocus(hWnd);

      }

      if(LOWORD(wParam) == FILE_OPEN) {

        OPENFILENAME ofn;
        char fileName[_MAX_PATH];
        strcpy(fileName,"");
        ofn.lStructSize       = sizeof(OPENFILENAME);
        ofn.hwndOwner         = hWnd;
        ofn.hInstance         = NULL;
        ofn.lpstrFilter       = "WML Files (*.wml)\0*.wml\0";
        ofn.lpstrCustomFilter = NULL;
        ofn.nMaxCustFilter    = 0;
        ofn.nFilterIndex      = 0;
        ofn.lpstrFile         = fileName; 
        ofn.nMaxFile          = _MAX_PATH;
        ofn.lpstrFileTitle    = NULL; 
        ofn.nMaxFileTitle     = _MAX_FNAME + _MAX_EXT;
        ofn.lpstrInitialDir   = NULL;
        ofn.lpstrTitle        = NULL;
        ofn.Flags             = OFN_HIDEREADONLY; 
        ofn.nFileOffset       = 0;
        ofn.nFileExtension    = 0;
        ofn.lpstrDefExt       = "wml";
        ofn.lCustData         = 0L;
        ofn.lpfnHook          = NULL;
        ofn.lpTemplateName    = NULL;

        if(GetOpenFileName(&ofn)) {
          
          String f("file:/");
          f.append(fileName);
          f.replaceWith('\\', '/');

          URI *uri = new URI(f.getPtr());
          ww->setSource(uri);

          updateWindow(ww->getUpdates());

        }

        InvalidateRect(content,NULL,false);
        UpdateWindow(content);

      }

      if(LOWORD(wParam) == buttonMiddle) {

        ww->pushedButton(WMLWindow::buttonOK);
        InvalidateRect(hWnd,NULL,false);
        UpdateWindow(hWnd);
        
      }

      if(LOWORD(wParam) == locationWindow) {

        if(HIWORD(wParam) == EN_SETFOCUS) {

          RegisterHotKey(hWnd, 0, 0, 13);

        }
        
        if(HIWORD(wParam) == EN_KILLFOCUS) {

          UnregisterHotKey(hWnd, 0);

        }

      }

      return 0;

    case WM_HOTKEY:

      if(HIWORD(lParam) == 13) {

        char *newLocationUrl = new char[250];
        SendMessage(locationText, WM_GETTEXT,(WPARAM) 250, (LPARAM) newLocationUrl);

        URI uri(newLocationUrl);
        ww->setSource(&uri);
      
        delete newLocationUrl;

        updateWindow(ww->getUpdates());
        InvalidateRect(content,NULL,false);
        UpdateWindow(content);
        SetFocus(hWnd);

      }

      return 0;

    case WM_DESTROY:
      delete ww;
      delete wi;
      delete mp;
      DeleteObject(hFont);
	    PostQuitMessage(0);
	    return 0;

    case WM_KEYDOWN:
      switch(wParam) {

      case VK_UP:
        SendMessage(content,WM_VSCROLL,SB_LINEUP,0);
        return 0;

      case VK_DOWN:
        SendMessage(content,WM_VSCROLL,SB_LINEDOWN,0);
        return 0;

      default:
        return 0;

      }
      return 0;

    case 0x020A:

      if(HIWORD(wParam) == 120)
        SendMessage(content,WM_VSCROLL,SB_PAGEUP,0);
      else
        SendMessage(content,WM_VSCROLL,SB_PAGEDOWN,0);
      
      return 0;

    default:
	    break;

   }
   return DefWindowProc(hWnd, message, wParam, lParam);

}

LRESULT CALLBACK ContentProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) {
  RECT* content;
  RECT main, rt;
  int cx, cy;
  double oldContentWQ, oldContentHQ;
  int scroll;

  switch (message) {
  
  case WM_SIZING:
    content = (RECT*) lParam;
    GetWindowRect(MainWindow,&main);

    oldContentWQ = contentWQ;
    oldContentHQ = contentHQ;
   
    contentWQ = (double) (content->right - content->left) / (double) (main.right - main.left);
    contentHQ = (double) (content->bottom - content->top) / (double) (main.bottom - main.top) / (1- BalkQuotient);

    cx = content->right - content->left;
    cy = content->bottom - content->top;

    POINT ul, lr;
    ul.x = (main.right  - main.left) / 2 - cx / 2; 
    lr.x = (main.right  - main.left) / 2 + cx / 2;
    ul.y = int(((double)(main.bottom - main.top) * (1 - BalkQuotient)) / 2 - (double)cy / 2);
    lr.y = int(((double)(main.bottom - main.top) * (1 - BalkQuotient)) / 2 + (double)cy / 2);

    if(lr.y > (int)((double)(main.bottom - main.top) * (1 - BalkQuotient) - 2 * WindowOffset)) {
      ul.y = 2 * WindowOffset;
      lr.y = (int)((double)(main.bottom - main.top) * (1 - BalkQuotient) - 2 * WindowOffset);
      contentHQ = oldContentHQ;
    }

    if(ul.x < 2 * WindowOffset) {
      ul.x = 2 * WindowOffset;
      lr.x = main.right - main.left - 2 * WindowOffset;
      contentWQ = oldContentWQ;
    }
    
    ClientToScreen(MainWindow, &ul);
    ClientToScreen(MainWindow, &lr);

    content->left = ul.x-4;
    content->right = lr.x-4; // probably the thickness of window 
    content->top
      = ul.y-15;
    content->bottom = lr.y-15;

    return true;

  case WM_VSCROLL:

    scroll = 0;

    switch (LOWORD(wParam)) {
    case SB_LINEUP:
      scroll=-10;
      break;
    case SB_LINEDOWN:
      scroll=10;
      break;
    case SB_PAGEUP:
      scroll=-40;
      break;
    case SB_PAGEDOWN:
      scroll=40;
      break;
    case SB_THUMBPOSITION:
      scroll = HIWORD(wParam) - GetScrollPos(hWnd,SB_VERT);
      break;
    case SB_THUMBTRACK:
      scroll = HIWORD(wParam) - GetScrollPos(hWnd,SB_VERT);
      break;

    }

    ww->scrollVertical(scroll);

    if(ww->getUpdates()->getFirstAction() != UpdateWindowList::actionNoMoreActions) {

      InvalidateRect(hWnd,NULL,false);
      UpdateWindow(hWnd);

    }

    return 0;

  case WM_HSCROLL:

    scroll = 0;

    switch (LOWORD(wParam)) {
    case SB_LINEUP:
      scroll=-10;
      break;
    case SB_LINEDOWN:
      scroll=10;
      break;
    case SB_PAGEUP:
      scroll=-20;
      break;
    case SB_PAGEDOWN:
      scroll=20;
      break;
    case SB_THUMBPOSITION:
      scroll = HIWORD(wParam) - GetScrollPos(hWnd,SB_HORZ);
      break;
    case SB_THUMBTRACK:
      scroll = HIWORD(wParam) - GetScrollPos(hWnd,SB_HORZ);
      break;

    }

    ww->scrollHorizontal(scroll);

    if(ww->getUpdates()->getFirstAction() != UpdateWindowList::actionNoMoreActions) {

      InvalidateRect(hWnd,NULL,false);
      UpdateWindow(hWnd);

    }

    return 0;

  case WM_SIZE:
    InvalidateRect(hWnd,NULL,false);
    return 0;

  case WM_PAINT:
    GetClientRect(hWnd,&rt);
    ww->setWindowDimensions(rt.right,rt.bottom);
    while(ww->getUpdates()->getFirstAction() != UpdateWindowList::actionNoMoreActions) {

      updateWindow(ww->getUpdates());
      GetClientRect(hWnd,&rt);
      ww->setWindowDimensions(rt.right,rt.bottom);

    }

    ww->drawContentWindow();
    break;

  case WM_MOUSEMOVE:
    ww->mouseMove(LOWORD(lParam), HIWORD(lParam));
    if(ww->getMousePointerShape())
      SetCursor(LoadCursor(NULL, MAKEINTRESOURCE(32649)));
    
    updateWindow(ww->getUpdates());
    return 0;

  case WM_LBUTTONUP:
    ww->mouseClick(LOWORD(lParam), HIWORD(lParam));
    updateWindow(ww->getUpdates());
    InvalidateRect(hWnd,NULL,false);
    UpdateWindow(hWnd);
    return 0;

  }
  return DefWindowProc(hWnd, message, wParam, lParam);
}

void updateWindow(UpdateWindowList* uwl) {

  int i;

  while(uwl->getFirstAction() != UpdateWindowList::actionNoMoreActions) {

    switch(uwl->getFirstAction()) {

    case (UpdateWindowList::actionSetOutputWindowText):
      SendMessage(GetDlgItem(MainWindow, outputWindow),WM_SETTEXT,0,(LPARAM) uwl->getFirstArgument()->getPtr());
      break;

    case (UpdateWindowList::actionClearOutputWindowText):
      SendMessage(GetDlgItem(MainWindow, outputWindow),WM_SETTEXT,0,(LPARAM) "");
      break;

    case (UpdateWindowList::actionURIChanged):
      SendMessage(GetDlgItem(MainWindow, locationWindow),WM_SETTEXT,0,(LPARAM) uwl->getFirstArgument()->getPtr());
      break;

    case (UpdateWindowList::actionAddButtonLabel):
      SendMessage(GetDlgItem(MainWindow, textLeft),CB_ADDSTRING,0,(LPARAM) uwl->getFirstArgument()->getPtr());
      SendMessage(GetDlgItem(MainWindow, textLeft),CB_SETCURSEL,0,0);
      break;

    case (UpdateWindowList::actionClearButtons):

      i = SendMessage(GetDlgItem(MainWindow, textLeft),CB_GETCOUNT,0,0);

      while(i != 0) {

        SendMessage(GetDlgItem(MainWindow, textLeft),CB_DELETESTRING,0,0);
        i = SendMessage(GetDlgItem(MainWindow, textLeft),CB_GETCOUNT,0,0);

      }
      SendMessage(GetDlgItem(MainWindow, textMiddle),WM_SETTEXT,0,(LPARAM)"Back");
      break;

    case (UpdateWindowList::actionHorizontalScrollBar):

      if(uwl->getFirstArgument()->equals("on"))
        ShowScrollBar(GetDlgItem(MainWindow, contentWindow), SB_HORZ, TRUE);
      if(uwl->getFirstArgument()->equals("off"))
        ShowScrollBar(GetDlgItem(MainWindow, contentWindow), SB_HORZ, FALSE);

      break;

    case (UpdateWindowList::actionHorizontalScrollBarRange):
      if(uwl->getFirstArgument()->isNumber()) {

        SCROLLINFO si;
        si.cbSize = sizeof(si); 
        si.fMask  = SIF_RANGE;
        si.nMin   = 0; 
        si.nMax   = uwl->getFirstArgument()->getInt();
        SetScrollInfo(GetDlgItem(MainWindow, contentWindow), SB_HORZ, &si, TRUE);

      }
      break;

    case (UpdateWindowList::actionHorizontalScrollBarPage):
      if(uwl->getFirstArgument()->isNumber()) {

        SCROLLINFO si;
        si.cbSize = sizeof(si); 
        si.fMask  = SIF_PAGE; 
        si.nPage  = uwl->getFirstArgument()->getInt();
        SetScrollInfo(GetDlgItem(MainWindow, contentWindow), SB_HORZ, &si, TRUE);

      }
      break;

    case (UpdateWindowList::actionHorizontalScrollBarPos):
      if(uwl->getFirstArgument()->isNumber()) {

        SCROLLINFO si;
        si.cbSize = sizeof(si); 
        si.fMask  = SIF_POS; 
        si.nPos   = uwl->getFirstArgument()->getInt();
        SetScrollInfo(GetDlgItem(MainWindow, contentWindow), SB_HORZ, &si, TRUE);

      }
      break;

    case (UpdateWindowList::actionVerticalScrollBar):

      if(uwl->getFirstArgument()->equals("on"))
        ShowScrollBar(GetDlgItem(MainWindow, contentWindow), SB_VERT, TRUE);
      if(uwl->getFirstArgument()->equals("off"))
        ShowScrollBar(GetDlgItem(MainWindow, contentWindow), SB_VERT, FALSE);

      break;

    case (UpdateWindowList::actionVerticalScrollBarRange):
      if(uwl->getFirstArgument()->isNumber()) {

        SCROLLINFO si;
        si.cbSize = sizeof(si); 
        si.fMask  = SIF_RANGE;
        si.nMin   = 0; 
        si.nMax   = uwl->getFirstArgument()->getInt();
        SetScrollInfo(GetDlgItem(MainWindow, contentWindow), SB_VERT, &si, TRUE);

      }
      break;

    case (UpdateWindowList::actionVerticalScrollBarPage):
      if(uwl->getFirstArgument()->isNumber()) {

        SCROLLINFO si;
        si.cbSize = sizeof(si); 
        si.fMask  = SIF_PAGE; 
        si.nPage  = uwl->getFirstArgument()->getInt();
        SetScrollInfo(GetDlgItem(MainWindow, contentWindow), SB_VERT, &si, TRUE);

      }
      break;

    case (UpdateWindowList::actionVerticalScrollBarPos):
      if(uwl->getFirstArgument()->isNumber()) {

        SCROLLINFO si;
        si.cbSize = sizeof(si); 
        si.fMask  = SIF_POS; 
        si.nPos   = uwl->getFirstArgument()->getInt();
        SetScrollInfo(GetDlgItem(MainWindow, contentWindow), SB_VERT, &si, TRUE);

      }
      break;

    case (UpdateWindowList::actionSetHandMousePointer):
      break;

    case (UpdateWindowList::actionSetNormalMousePointer):
      break;

    case (UpdateWindowList::actionBackButton):
      EnableWindow(GetDlgItem(MainWindow, textMiddle),
                               uwl->getFirstArgument()->equals("on") ? 
                               TRUE : FALSE);
      break;

    case (UpdateWindowList::actionForwardButton):
      EnableWindow(GetDlgItem(MainWindow, textRight),
                               uwl->getFirstArgument()->equals("on") ? 
                               TRUE : FALSE);
      break;
    default:
      break;

    }
    uwl->deleteFirst();


  }

}
