/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: ContentSink.cpp,v 1.10 2001/03/14 15:55:14 wouter Exp $ 
 */

#include <stdio.h>
#include <string.h>

#include "Monitor.h"
#include "URI.h"
#include "OutOfMemoryException.h"
#include "ContentSinkResult.h"
#include "ContentSink.h"
#include "HTTP.h"
#include "About.h"

/**
 * Creates the ContentSink.
 */
ContentSink::ContentSink() {

  httpHandler = new HTTP();
  aboutHandler = new About();

}

/**
 * Destroys the content sink.
 */
ContentSink::~ContentSink() {

  delete httpHandler;
  delete aboutHandler;

}

bool ContentSink::test() {

  ContentSink cs;
  
  URI u("http://www.catchy.net/wap/index.wml");

  ContentSinkResult* csr = cs.get(&u, 0);

  char *t;
  unsigned int a;

  csr->getContent((void **)&t,&a);

  if(t[0] != '<')
    return false;

  if(t[1] != '?')
    return false;

  if(t[2] != 'x')
    return false;
  
  delete csr;

  return true;

}

/**
 * Retrieves some data from the given URI.
 * @throws OutOfMemoryException when some memory could not be allocated.
 * @param u the URI to fetch.
 * @param m the Monitor that receives progress reports.
 */
ContentSinkResult *ContentSink::get(URI *u, Monitor *m) {
  ContentSinkResult *result = new ContentSinkResult();
  result->setMimeType("text/vnd.wap.wml");

  if(u->isValid()) {

    /* URI is valid. */

    if(u->getScheme()) {

      if(strcmp(u->getScheme(), "http") == 0) {

        /* Get content via HTTP. */

        httpHandler->get(u, m, result);

      } else if(strcmp(u->getScheme(), "about") == 0) {

        /* Generate appropriate about: content. */

        aboutHandler->get(u, m, result);

      } else if(strcmp(u->getScheme(), "file") == 0) {

        /* Get content from local filesystem. */

        if(u->getPath()) {
        
          String s("");

#ifndef WIN32
          s.append("/");
#endif

          s.append(u->getAbsolutePath());

          FILE *f = fopen(s.getPtr(), "rb");
        
          if(f) {

            /* Get file's length. */
          
            int curPos = ftell(f);
            fseek(f, 0, SEEK_END);
            int fileLength = ftell(f);
            fseek(f, curPos, SEEK_SET);

            char *buffer = new char[fileLength];

            if(buffer) {

              fread(buffer, 1, fileLength, f);
              //           buffer[fileLength] = '\0';
              result->setContent(buffer, fileLength);

            } else {

              result->setContent("<wml><card><p>Insufficient memory to open file.</p></card></wml>");
            
            }

            fclose(f);

          } else {

            result->setContent("<wml><card><p>Failed to open file.</p></card></wml>");
          
          }
        
        } else {

        }

      } else {

        result->setContent("<wml><card><p>Unknown URI scheme.</p></card></wml>");

      }

    }

  }

  return result;

}


