/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: String.cpp,v 1.16 2001/03/08 16:18:50 wouter Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "String.h"

/** 
 * Constructs a String object, containing an empty string.
 */
String::String() {

  theString = NULL;

}

/** 
 * Constructs a new String object containing the same string as the 
 * character array.
 * @param s  the character array to be converted into the
 * String Object
 */
String::String(char *s) {

  theString = newString(s);

}

/** 
 * Constructs a new String object containing the same string as the
 * character array.
 * @param  string  the character array to be converted into the
 * String Object
 */
String::String(const char *s) {

  theString = newString(s);

}

/** 
 * Constructs a new String object from an array.
 * @param  string  the character array to be converted into the
 * String Object
 */
String::String(const char *buffer, size_t length) {

  theString = new char[length + 1];
  
  memcpy(theString, buffer, length);
  theString[length] = '\0';

}

/** 
 * Constructs a new String object containing the same string as 
 * another String object.
 * @param s  the string from which the new String object must
 * be built.
 */
String::String(String *s) {

  if(s == NULL) {

    theString = NULL;
    return;

  }

  theString = newString(s->getPtr());

}

/**
 * Allocates a new string containing the same string as source
 * used by String(String *s) and String(char *s)
 * @param source  the string from which to make a new string
 * @return a pointer to a newly allocated string
 */
char *String::newString(const char *source) {

  char *dest;

  if(source == NULL)
    return NULL;

  int stringLength = strlen(source);

  if(stringLength == 0)
    return NULL;

  dest = new char[stringLength + 1];  /* allocate enough memory */
  strcpy(dest, source);               /* copy source */
  dest[stringLength] = '\0';          /* make string null-terminated */
  return dest;

}

/**
 * Delete the object
 */
String::~String() {
  
  delete theString;

}

/** 
 * This function tests the code of this class
 * @return true when all tests succeed, otherwise false
 */
bool String::test() {

  String s;
  String t("test");

  /* testing append(String*)/equals(String*) */
  s.append(&t);
  if(!s.equals("test"))
    return false;
  if(!s.equals(&t))
    return false;

  /* testing append(char*)/getPtr()/clear() */
  s.clear();
  s.append(t.getPtr());
  if(!s.equals("test"))
    return false;

  s.clear();
  if(s.getPtr() != "")
    return false;

  /* testing append(char) */
  s.append('a');
  if(!s.equals("a"))
    return false;

  s.append('b');
  if(!s.equals("ab"))
    return false;

  s.clear();

  /* testing append(int) */
  s.append(3);
  if(!s.equals("3"))
    return false;

  s.append(4);
  if(!s.equals("34"))
    return false;

  s.clear();

  /* testing append(double) */
  s.append(3.4);
  if(!s.equals("3.400"))
    return false;

  s.append(4.123);
  if(!s.equals("3.4004.123"))
    return false;

  s.clear();

  /* testing equals(char*) */

  if(s.equals("") == false)
    return false;

  if(s.equals("a") == true)
    return false;

  s.append("a");

  if(s.equals("") == true)
    return false;

  if(s.equals("a") == false)
    return false;

  s.clear();

  /* testing getLength() */

  if(s.getLength() != 0)
    return false;

  s.append("123");

  if(s.getLength() != 3)
    return false;

  s.clear();

  /* testing isNumber()/isHexNumber() */

  if(s.isNumber())
    return false;

  if(s.isHexNumber())
    return false;

  s.append("123");

  if(!s.isNumber())
    return false;

  if(!s.isHexNumber())
    return false;

  s.append("123abf");

  if(s.isNumber())
    return false;

  if(!s.isHexNumber())
    return false;

  s.clear();

  /* testing getInt()/getHexInt() */

  if(s.getInt() != 0)
    return false;

  if(s.getHexInt() != 0)
    return false;

  s.append("3");

  if(s.getInt() != 3)
    return false;

  if(s.getHexInt() != 3)
    return false;

  s.append("a");

  if(s.getInt() != 0)
    return false;

  if(s.getHexInt() != 58) /* hex(58) = 3a */
    return false;

  s.clear();

  /* testing removeFirstCharacter()/getFirstCharacter() */

  if(s.getFirstCharacter() != '\0')
    return false;
  
  s.removeFirstCharacter();

  if(!s.equals(""))
    return false;

  s.append("ab");

  if(s.getFirstCharacter() != 'a')
    return false;
  
  s.removeFirstCharacter();

  if(s.getFirstCharacter() != 'b')
    return false;
  
  s.removeFirstCharacter();

  if(!s.equals(""))
    return false;

  s.clear();

  /* testing removeLastCharacter()/getLastCharacter() */

  if(s.getLastCharacter() != '\0')
    return false;
  
  if(s.removeLastCharacter() != '\0')
    return false;

  if(!s.equals(""))
    return false;

  s.append("ab");

  if(s.getLastCharacter() != 'b')
    return false;
  
  if(s.removeLastCharacter() != 'b')
    return false;

  if(s.getLastCharacter() != 'a')
    return false;
  
  if(s.removeLastCharacter() != 'a')
    return false;

  if(!s.equals(""))
    return false;

  /* testing substring() */

  t.clear();

  if(!s.substring(&t))
    return false;

  s.append("abcd");
  t.append("bc");

  if(!s.substring(&t))
    return false;

  t.append("e");

  if(s.substring(&t))
    return false;

  s.clear();
  t.clear();
  
  /* testing replaceWith() */

  s.append("abba");
  s.replaceWith('b','n');
  s.replaceWith('a','o');
  if(!s.equals("onno"))
    return false;

  s.clear();

  /* testing removeAll() */

  s.append("anagram");
  s.removeAll('a');
  if(!s.equals("ngrm"))
    return false;

  s.clear();

  /* testing makeDosText() */

  s.append("line1\nline2\nline3\n");

  int i = s.getLength();

  s.makeDosText();

  if(s.getLength() != i + 3)
    return false;

  return true;

}

/** 
 * This function must be used to get acces to the pointer the 
 * String object internally uses. This function never returns
 * NULL. The returned string is always null-terminated. Typical use:
 *
 *     String *s = new String("hallo");
 *     printf(s->getPtr());
 * 
 * Don't expect the pointer to be always the same for an instance of
 * this object.
 * @return     the pointer the String object internally uses
 */
char *String::getPtr() const {

  if(theString != NULL)
    return theString;
  return "";

}

/**
 * Makes the string empty, in other words: afterwards the string 
 * contains "".
 */
void String::clear() {
	
  if(theString) {

    delete theString;
    theString = NULL;

  }

}

/** 
 * Appends the String object to the end of the string.
 *
 * @param  s  the string to be appended
 */
void String::append(String *s) {

  append(s->getPtr());

}

/** 
 * Appends the character array to the end of the string.
 *
 * @param  s  the string to be appended
 */
void String::append(const char *s) {

  if(s == NULL) return; 
  if(theString != NULL) {

    char *begin = theString;
    int totalLength = strlen(begin) + strlen(s) + 1;
    theString = new char[totalLength];
    strcpy (theString, begin);         /* copy original to string */
    strcat (theString, s);             /* copy the new part to the string */
    theString[totalLength - 1] = '\0'; /* make string null-terminated */
    delete begin;

  } else {

    int stringLength = strlen(s);
    theString = new char[stringLength + 1];
    strcpy(theString, s);
    theString[stringLength] = '\0';    /* make string null-terminated */

  }
  if(strlen(theString) == 0) clear();
  /* if the final string is empty: delete the pointer */

}

/** 
 * Appends the character to the end of the string.
 *
 * @param  c  the character to be appended
 */
void String::append(char c) {

  if(theString != NULL) {

    char *begin = theString;
    int totalLength = strlen(begin) + 2; /* the new character + 0-terminator */
    theString = new char[totalLength];
    strcpy (theString,begin);   /* copy original to string */
    theString[totalLength-2] = c;
    theString[totalLength-1] = '\0';
    delete begin;

  } else {

    theString = new char[2];
    theString[0] = c;
    theString[1] = '\0'; /* make string null-terminated */

  }

}

/** 
 * Appends the integer to the end of the string.
 *
 * @param  integer the integer to be appended
 */
void String::append(int integer) {

  char string[((sizeof(int)*8)/3)+2];  /* allocate sufficient amount of 
                                          decimal places */

  sprintf(string,"%d",integer); /* convert the integer to a string */
	
  if(theString != NULL) {

    char *begin = theString;
    int totalLength = strlen(begin) + strlen(string) + 1;
    theString = new char[totalLength];
    strcpy (theString, begin);         /* copy original to string */
    strcat (theString, string);        /* copy the integer to the string */
    theString[totalLength - 1] = '\0'; /* make string null-terminated */
    delete begin;

  } else {

    int stringLength = strlen(string);
    theString = new char[stringLength + 1];
    strcpy(theString, string);
    theString[stringLength] = '\0';    /* make string null-terminated */

  }

}

/** 
 * Appends the double to the end of the string. Three decimals
 * after the '.' are shown. For example: 1.234
 *
 * @param  d the double to be appended
 */
void String::append(double d) {

  char string[100];  /* allocate sufficient amount of 
                        decimal places */

  sprintf(string,"%.3f",d); /* convert the integer to a string */
	
  if(theString != NULL) {

    char *begin = theString;
    int totalLength = strlen(begin) + strlen(string) + 1;
    theString = new char[totalLength];
    strcpy (theString, begin);         /* copy original to string */
    strcat (theString, string);        /* copy the integer to the string */
    theString[totalLength - 1] = '\0'; /* make string null-terminated */
    delete begin;

  } else {

    int stringLength = strlen(string);
    theString = new char[stringLength + 1];
    strcpy(theString, string);
    theString[stringLength] = '\0';    /* make string null-terminated */

  }

}

/** 
 * Compares the string in the argument to the String Object.
 *
 * @param  s the string to compare this String Object against
 * @return true  if the two strings represent the same sequence
 *               of characters, otherwise false
 */
bool String::equals(const char *s) const{

  if(s == NULL) 
    return (getLength() == 0);
  if(theString == NULL) 
    return (strcmp(s, "") == 0);
  if(strcmp(theString, s) == 0)
    return true;
  
  return false;

}

/** 
 * Compares the string in the argument to the String Object.
 *
 * @param s  the string to compare this String Object against
 * @return true if the two strings represent the same sequence of
 *         characters, otherwise false
 */
bool String::equals(const String *s) const {

  return equals(s->getPtr());

}

/**
 * Determines the length of the string, in other words the number
 * of characters in the string.
 *
 * @return the length of the string
 */
int String::getLength() const {

  if(theString) 
    return strlen(theString);
  return 0;

}

/**
 * Determines whether the character in the argument is a number 
 * (0-9).
 *
 * @param c  a character
 * @return true if c is a number
 */
bool String::isNumber(char c) {

  return (c >= '0' && c <= '9');

}

/** 
 * Determines whether the character in the argument is a number
 * (0-9) or a character (a-f) or (A-F)
 *
 * @param c  a character
 * @return true if c is a hexadecimal number
 */
bool String::isHexNumber(char c) {

  if(c >= '0' && c <= '9')
    return true;
  if(c >= 'a' && c <= 'f')
    return true;
  if(c >= 'A' && c <= 'F')
    return true;
  return false;

}

/**
 * Determines whether the string a number.
 *
 * @return true if the string is a number
 */
bool String::isNumber() const {

  if(getLength() < 1)
    return false;
  int i = getLength() - 1;

  for(;i >= 0; i--)
    if(!isNumber(theString[i]))
      return false;
  
  return true;
}

/** 
 * Determines whether the string a hexadecimal number.
 *
 * @return true if the string is a hexadecimal number
 */
bool String::isHexNumber() const {

  if(getLength() < 1)
    return false;
  int i = getLength() - 1;

  for(;i >= 0; i--)
    if(!isHexNumber(theString[i]))
      return false;
  
  return true;

}

/** 
 * Returns the integer value of the String, return value is 0
 * when the string is not a number or undefined when the number
 * is too big to fit into an integer.
 *
 * @return the integer value of the string
 */
int String::getInt() const {

  if(!isNumber())
    return 0;
  
  return atoi(theString);
  
}

/** 
 * Returns the hexadecimal integer value of the String, return value
 * is 0 when the string is not a hexadecimal number or undefined when
 * the number is too big to fit into an integer.
 *
 * @return the integer value of the string
 */
int String::getHexInt() const{

  if(!isHexNumber())
    return 0;
  return strtol(theString,NULL,16);
  
}

/**
 * Returns the first character of the string.
 * @return the first character of the string
 */
char String::getFirstCharacter() const {

  if(theString == 0)
    return '\0';
  return theString[0];

}

/**
 * Removes the first character of the string.
 */
void String::removeFirstCharacter() {

  if(theString == 0)
    return;

  if(getLength() == 0)
    return;

  if(getLength() == 1) {

    clear();
    return;

  }

  char *temp = newString(theString + 1);
  delete theString;
  theString = temp;

}

/** 
 * Returns the last character from the string
 *
 * @return the last character
 */
char String::getLastCharacter() const{

  if(getLength() < 1)
    return 0;
  return theString[getLength()-1];

}

/** 
 * Removes the last character from the string
 *
 * @return the removed character
 */
char String::removeLastCharacter() {
  
  if(getLength() < 1)
    return 0;

  char c = theString[getLength()-1];
  theString[getLength()-1] = 0;
  if(getLength() == 0)
    clear();

  return c;

}

/**
 * Determines whether the string in the argument is a substring of
 * this string
 *
 * @return true iff it's a substring
 */ 
bool String::substring(String *s) const {

  if(s == NULL)
    return false;
  if(theString == NULL)
    return (s->getLength() == 0);
  if(strstr(theString,s->getPtr()))
    return true;

  return false;

}

/**
 * Replaces all occurences of c1 with c2.
 * @param c1 the character to replace
 * @param c2 the character to replace c1 with
 */
void String::replaceWith(char c1, char c2) {

  int i;

  for(i = getLength() - 1; i >= 0; i--)
    if(theString[i] == c1)
      theString[i] = c2;
   
}

/**
 * Removes all occurences of the character in the argument
 * @param c the character which must be removed
 */
void String::removeAll(char c) {

  if(c == '\0')
    return;
  while(removeOne(c));
  
}

/**
 * Removes one occurence of c.
 * @param c the character to remove
 * @return true when there was a occurence removed, 
 *         false when c was not found in the string
 */
bool String::removeOne(char c) {

  int i;

  for(i = 0; i < getLength(); i++) {
    if(theString[i] == c) {

      char *s = &(theString[i+1]);
      theString[i] = '\0';
      char *t = theString;

      theString = NULL;
      append(t);
      append(s);
      delete t;

      return true;
    }
  }

  return false;

}

/**
 * Replaces all occurences of '\n' in the string with '\r\n'.
 */
void String::makeDosText() {

  String newString;
  char c;

  while(getLength()) {

    c = getFirstCharacter();
    if(c == '\n')
      newString.append("\r\n");
    else
      newString.append(c);

    removeFirstCharacter();
  }

  clear();
  append(&newString);

}
