/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: String.h,v 1.13 2001/03/07 11:47:39 pascal Exp $
 */

#ifndef __STRING_H__
#define __STRING_H__

#include <stdlib.h>

/** 
 * A class representing a string.
 * For example:
 * \code
 *  String *s = new String("hallo");
 *  printf(s->getPtr());
 * \endcode
 */
class String {

public:

  String();
  String(char *source);
  String(const char *source);
  String(const char *buffer, size_t length);
  String(String *source);
  ~String();
  static bool test();
  char *getPtr() const;
  void clear();
  void append(String *string);
  void append(const char *string);
  void append(char c);
  void append(int integer);
  void append(double d);
  bool equals(const char *s) const;
  bool equals(const String *s) const;
  int getLength() const;
  static bool isNumber(char c);
  static bool isHexNumber(char c);
  bool isNumber() const;
  int getInt() const;
  bool isHexNumber() const;
  int getHexInt() const;
  char getLastCharacter() const;
  char removeLastCharacter(); 
  bool substring(String *s) const;
  char getFirstCharacter() const;
  void removeFirstCharacter();
  void replaceWith(char c1, char c2);
  void removeAll(char c);
  void makeDosText();

private:

  bool removeOne(char c);
  char *newString(const char *source);

  /* member data */
  /** the pointer to the string */
  char *theString;

};



#endif

