/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: Socket.cpp,v 1.3 2001/03/12 16:35:03 wouter Exp $ 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <winsock2.h>

#include "../AbstractSocket.h"
#include "../Socket.h"
#include "../Exception.h"

Socket::Socket(const char *hostname, unsigned short port) : AbstractSocket() {

  /* Create a socket. */

  WORD wVersionRequested;
  WSADATA wsaData;
  int err;
 
  wVersionRequested = MAKEWORD( 1, 0 );
 
  err = WSAStartup( wVersionRequested, &wsaData );
  if ( err != 0 ) {
      throw Exception("Wrong WinSock version.");
      return;
  }
 
  /* Confirm that the WinSock DLL supports 2.2.*/
  /* Note that if the DLL supports versions greater    */
  /* than 2.2 in addition to 2.2, it will still return */
  /* 2.2 in wVersion since that is the version we      */
  /* requested.                                        */
 
  if ( LOBYTE( wsaData.wVersion ) != 1 ||
          HIBYTE( wsaData.wVersion ) != 0 ) {

      throw Exception("Wrong WinSock version (2).");
      WSACleanup( );

      return; 
  }
 

  socketDescriptor = socket(AF_INET, SOCK_STREAM, 0);

  if(socketDescriptor != -1) {

    /* Do a hostname lookup. */

    struct hostent *hostent;
    hostent = gethostbyname(hostname);
    
    if(hostent) {
      
      /* Try to connect the socket. */

      struct sockaddr_in sin;
      sin.sin_family = (short) hostent->h_addrtype;
      sin.sin_port = htons((unsigned short) port);
      sin.sin_addr.s_addr = *(unsigned long *) hostent->h_addr;
      
      if(connect(socketDescriptor, (sockaddr *) &sin, sizeof(sin)) == -1) 
        throw Exception("Connection failed.");
      
    } else {

      char *s = (char *) malloc(strlen("Unknown host: ") + 
                                strlen(hostname) + 1);

      if(s) {

        sprintf(s, "Unknown host: %s", hostname);
        throw Exception(s);

      } else {

        throw Exception("Unknown host");

      }

    }
    
  } else
    throw Exception("Cannot create connection.");

}

Socket::~Socket() {

  closesocket(socketDescriptor);
  WSACleanup();

}

size_t Socket::readNative(void *buffer, size_t count) {

  return recv(socketDescriptor, (char *)buffer, count, 0);

}

size_t Socket::writeNative(const void *buffer, size_t count) {

  return send(socketDescriptor, (char *)buffer, count, 0);

}
