/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: IntList.cpp,v 1.13 2001/01/27 14:36:40 wouter Exp $
 */

#include <malloc.h>
#include <stdlib.h>

#include "IntList.h"

/** 
 * Constructs a new IntList object.
 */
IntList::IntList() {

  elements = NULL;
  size = 0;

}

/** 
 * Constructs a new IntList object that is the same as the list in the
 * argument.
 * @param il the object to be copied
 */
IntList::IntList(IntList* il) {

  size = il->getSize();
  elements = (int *)malloc(size * sizeof(int));

  unsigned int i;
  int *otherElements = il->getArray();

  for(i = 0; i < size; i++)
    elements[i] = otherElements[i];

}

/**
 * Delete the object
 */
IntList::~IntList() {

  delete elements;

}

/**
 * This function tests the code of this class
 *
 * @return  true when all tests succeed, otherwise false
 */
bool IntList::test() {

  IntList il2;

  il2.add(1);
  il2.add(2);
  il2.add(3);

  IntList il(&il2);

  if(il.getSize() != 3) 
    return false;

  if(il.getFirstAdded() != 1) 
    return false;  
  if(il.getLastAdded()  != 3) 
    return false;

  il.removeLastAdded();

  if(il.getSize() != 2) 
    return false;

  if(il.getFirstAdded() != 1) 
    return false;  
  if(il.getLastAdded()  != 2) 
    return false;

  il.removeFirstAdded();

  if(il.getSize() != 1) 
    return false;

  if(il.getFirstAdded() != 2) 
    return false;  
  if(il.getLastAdded()  != 2) 
    return false;

  il.removeFirstAdded();

  if(il.getSize() != 0) 
    return false;

  if(il.getFirstAdded() != 0) 
    return false;  
  if(il.getLastAdded()  != 0) 
    return false;

  il.add(4);

  if(il.getSize() != 1) 
    return false;

  if(il.getFirstAdded() != 4) 
    return false;  
  if(il.getLastAdded()  != 4) 
    return false;

  il.removeFirstAdded();

  if(il.getSize() != 0) 
    return false;

  if(il.getFirstAdded() != 0) 
    return false;  
  if(il.getLastAdded()  != 0) 
    return false;

  il.removeLastAdded();
  il.removeFirstAdded();

  if(il.getSize() != 0) 
    return false;
  
  return true;

}

/**
 * Empties the list.
 */
void IntList::clear() {

  delete elements;
  elements = NULL;
  size = 0;

}

/**
 * Adds an interger to the integer list
 *
 * @param   i   the integer that must be added
 */
void IntList::add(int i) {

  elements = (int*)realloc(elements,(size+1)*sizeof(int));

  elements[size] = i;
  
  size++;

}

/**
 * Removes the last added integer from the list.
 */
void IntList::removeLastAdded() {

  if(size == 0) 
    return;
  if(size == 1) {
    delete elements;
    elements = NULL;
    size = 0;
    return;
  }

  size--;

  elements = (int*)realloc(elements,size * sizeof(int));

}

/**
 * Removes the first added integer from the list.
 */
void IntList::removeFirstAdded() {

  if(size == 0) 
    return;
  if(size == 1) {
    delete elements;
    elements = NULL;
    size = 0;
    return;
  }

  size--;

  int *oldElements = elements;
  elements = (int*) malloc((size) * sizeof(int));
  unsigned int i;
  for(i = 0; i < size; i++)
    elements[i] = oldElements[i + 1];

  delete oldElements;

  return;

}

/**
 * Returns the last added integer to the list, when the list is empty,
 * returns 0.
 * @return the last added integer to the list, when the list is empty,
 * returns 0.
 */
int IntList::getLastAdded() {

  if(size == 0) 
    return 0;
  return elements[size - 1];

}

/**
 * Returns the first added integer to the list, when the list is empty,
 * returns 0.
 * @return the first added integer to the list, when the list is empty,
 * returns 0.
 */
int IntList::getFirstAdded() {

  if(size == 0) 
    return 0;
  return elements[0];

}

/**
 * Returns the size of the list.
 * @return the size of the list
 */
unsigned int IntList::getSize() {

  return size;

}

/**
 * returns the used array.
 * @return the used array
 */
int *IntList::getArray() {

  return elements;

}

