/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLAnchorRectangleList.cpp,v 1.8 2001/01/27 14:36:40 wouter Exp $
 */

#include <stdlib.h>

#include "WMLAnchorRectangleList.h"
#include "../lib/String.h"

/** 
 * Constructs a new WindowsInterface object.
 */
WMLAnchorRectangleList::WMLAnchorRectangleList() {

  next = NULL;
  href = NULL;

}

/**
 * Delete the object.
 */
WMLAnchorRectangleList::~WMLAnchorRectangleList() {

  delete next;

}

/**
 * This function tests the code of this class
 *
 * @return      true when all tests succeed, otherwise false
 */
bool WMLAnchorRectangleList::test() {

  String s("http://www.somewhere.com/index.wml");
  WMLAnchorRectangleList warl;
  
  warl.add(8,8,9,9,&s);
  warl.add(14,14,16,16,&s);

  if(warl.getLinkTarget(15,15) != &s) return false;
  if(warl.getLinkTarget(7,7) != NULL) return false;
  warl.clear();

  return true;

}

/** 
 * Adds a rectangle to the list
 * @param x1   the left side of the rectangle
 * @param y1   the upper side of the rectangle
 * @param x2   the right side of the rectangle
 * @param y2   the lower side of the rectangle
 * @param link the link
 */
void WMLAnchorRectangleList::add(int x1, int y1, int x2, int y2, String *link) {

  if(next == 0) { /* this is the last element */

    upperLeft.x = x1;
    lowerRight.x = x2;
    upperLeft.y = y1;
    lowerRight.y = y2;
    href = link;  /* notice: the link is not copied, so this is a 
                     pointer in the parsetree */
    
    next = new WMLAnchorRectangleList(); 
        /* next is now the last element */

  } else {

    next->add(x1,y1,x2,y2,link); 
        /* push the element to the next position */
  }

}

/** 
 * Clears the entire list
 */
void WMLAnchorRectangleList::clear() {

  delete next;
  next = NULL;

}

/** 
 * Returns the href of the anchor located at x,y. When there is no 
 * anchor in that position in the list, returns NULL.
 * @param x the x coordinate
 * @param y the y coordinate
 * @return the href of the anchor located at x,y. When there is no
 * anchor in that position in the list, returns NULL.
 */
String* WMLAnchorRectangleList::getLinkTarget(int x, int y) {

  if(next == NULL)
    return 0;

  if(upperLeft.x <= x  &&
     x <= lowerRight.x &&
     upperLeft.y <= y  &&
     y <= lowerRight.y   )
      return href;

  return next->getLinkTarget(x,y);

}
