/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *      - Redistributions of source code must retain the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this
 *      software must display the following acknowledgement: This
 *      product includes software developed by catchy.net and its
 *      contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors
 *      may be used to endorse or promote products derived from this
 *      software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: WMLParser.h,v 1.9 2001/01/27 14:36:40 wouter Exp $ 
 *
 * Reviewed 4-12-2000 ph (naming, documentation)
 */

#ifndef __WMLPARSER_H__
#define __WMLPARSER_H__

class String;
class XMLBody;
class XMLElement;

#include "DTDDataStructures.h"

/** 
 * This class represents a WML Parser.
 * Typical use:
 * \code
 *  WMLParser *wp = new WMLParser(source);
 *  if(!wp->getErrorMessage())
 *    parseTree = new XMLElement(parser.getRootElement());
 *  else
 *    s = wp->getErrorMessage();
 *  delete wp;
 * \endcode
 */
class WMLParser {

public:
  WMLParser(String *source);
  ~WMLParser();
  static bool test();
  XMLElement *getRootElement();
  String *getErrorMessage();
  void getParseTreeAsString(String *s);

private:
  static bool test(char* source, bool shouldParse);
	void applyTemplateToCards();
  void generateError(char *err, XMLElement *xe);
  void generateExpectedError(XMLElement *body, int lineNumber, DTDDataStructures::BodyList *ol, int index, String *insteadOf);
  void generateRequiredAttributeError(XMLElement *xe, String *attributeName);
  void generateWrongAttributeValueError(XMLElement *xe, String *attributeName, String *attributeValues, String *insteadOf);
  void makeErrorText(String *s, char *message, XMLElement *xe);

  void checkElementWithDTD(XMLElement *xe);
  void checkAttributesWithDTD(XMLElement *xe);
  void checkAttributeTypeWithDTD(XMLElement *xe, String *attrName, String *attrValue, DTDDataStructures::AttributeType at);
  void replaceEntities(String *cdata, int startLineNumber, int endLineNumber);
  void replaceEntity(char *entity, String *pcdata, int startLineNumber, int endLineNumber);

  static bool checkBodyForDoubleTasks(XMLBody *xb);

  DTDDataStructures::BodyList           *getBodyList(char *tag);
  DTDDataStructures::RequiredAttributes *getRequiredAttributes(char *tag);
  DTDDataStructures::DefaultAttributes  *getDefaultAttributes(char *tag);
  DTDDataStructures::AllAttributes      *getAllAttributes(char *tag);

  /* member data */
  /** the error message */
  String *errorMessage;
  
  /** the top element of the parse tree */
  XMLElement *topElement;

  /** MaxReferenceLength = the max length of a reference (for example
      &gt; has length 2)*/
  enum { 
    maxReferenceLength = 10 
  };

};


#endif

