/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: XMLBody.h,v 1.7 2001/01/27 14:36:40 wouter Exp $
 *
 * Reviewed 4-12-2000 ph (enum, naming)
 */

#ifndef __XMLBODY_H__
#define __XMLBODY_H__

class String;
class XMLBody;
class XMLCharData;
class XMLElement;
class XMLParser;
class XMLTokenizer;

/** 
 * This class represents the body of a xml element. Example:
 * \code
 *  <link show="on"> bla bla <link2> bla </link2> </link></code>
 * \endcode
 * In this example 
 * \code
 *  bla bla <link2> bla </link2>
 * \endcode
 * is the body of the "link" tag.
 * 
 * Typical use:
 * \code
 *  XMLBody *xb= new XMLBody();
 *  xb->parse(tag, tokenizer, parser);
 * \endcode
 */
class XMLBody {

public:

  /** These are the possible content types. Element means the 
      body element is an element (<tag> blabla </tag>), 
      characterData means it is character data (blabla).
  */
  enum contentTypes {
    typeElement,
    typeCharacterData,
    typeNothing
  };

  XMLBody();
  XMLBody(XMLBody *xb);
  ~XMLBody();
  static bool test();
  void append(XMLBody *xb);
  void parse(String *tag, XMLTokenizer *tokenizer, XMLParser *parser);
  contentTypes getContentType();
  XMLElement *getElement();
  XMLBody *getElement(char *name);
  XMLCharData *getCharacterData();
  String *getCharacterDataAsString();
  XMLBody *getNextXMLBody();
  void getParseTreeAsString(int depth, String *s);

private:
  /* member data */

  /** the content type of this object, characterData or element*/
  contentTypes contentType;
  
  /** the element when the contentType = typeElement otherwise NULL */
  XMLElement *element;
  
  /** the characterdata when contenType = typeCData otherwise NULL */
  XMLCharData *characterData;
  
  /** the next element of the body */
  XMLBody *nextElement;
  
};



#endif
