/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptLibraryFloat.h,v 1.6 2001/01/27 14:36:41 wouter Exp $
 */

#ifndef __WMLSCRIPTLIBRARYFLOAT_H__
#define __WMLSCRIPTLIBRARYFLOAT_H__

#include "WMLScriptLibrary.h"

#define FUNC_INT 0
#define FUNC_FLOOR 1
#define FUNC_CEIL 2
#define FUNC_POW 3
#define FUNC_ROUND 4
#define FUNC_SQRT 5
#define FUNC_MAXFLOAT 6
#define FUNC_MINFLOAT 7

/**
 * This class implements the Float library as specified
 * by WAP-194-WMLScript Standard Libraries Specification [June 2000].
 * See the WAP-194 conformance document for notes regarding
 * floating point compatibility.
 */
class WMLScriptLibraryFloat : public WMLScriptLibrary {
 public:
  WMLScriptLibraryFloat();
  ~WMLScriptLibraryFloat();

  int getFunctionID(const char *name) const;
  WMLScriptValueObject *call(int id, WMLScriptFunctionParameters *parameters);
  const char *getName() const;
  
 private:
  WMLScriptValueObject *funcInt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcFloor(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcCeil(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcPow(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcRound(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcSqrt(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcMaxFloat(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcMinFloat(WMLScriptFunctionParameters *parameters);
};

#endif


