/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptLibraryString.cpp,v 1.6 2001/01/27 14:36:41 wouter Exp $
 */

#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <math.h>

#include "WMLScriptValueObject.h"
#include "WMLScriptFunctionParameters.h"
#include "WMLScriptLibraryString.h"
#include "WMLScriptWrongArityException.h"
#include "WMLScriptUnknownLibraryFunctionException.h"
#include "WMLScriptOutOfMemoryException.h"

/**
 * Creates the library.
 */
WMLScriptLibraryString::WMLScriptLibraryString() {
}

/**
 * Destroys the library.
 */
WMLScriptLibraryString::~WMLScriptLibraryString() {
}

/**
 * Returns the function ID of a given function,
 * or -1 if the function does not exist.
 * @param name the function name.
 * @return the ID of the given function or -1 if the function
 * does not exist.
 */
int WMLScriptLibraryString::getFunctionID(const char *name) const {
  if(strcmp(name, "length") == 0) {
    return FUNC_LENGTH;
  } else if(strcmp(name, "isEmpty") == 0) {
    return FUNC_ISEMPTY;
  } else if(strcmp(name, "charAt") == 0) {
    return FUNC_CHARAT;
  } else if(strcmp(name, "subString") == 0) {
    return FUNC_SUBSTRING;
  } else if(strcmp(name, "find") == 0) {
    return FUNC_FIND;
  } else if(strcmp(name, "replace") == 0) {
    return FUNC_REPLACE;
  } else if(strcmp(name, "elements") == 0) {
    return FUNC_ELEMENTS;
  } else if(strcmp(name, "elementAt") == 0) {
    return FUNC_ELEMENTAT;
  } else if(strcmp(name, "removeAt") == 0) {
    return FUNC_REMOVEAT;
  } else if(strcmp(name, "replaceAt") == 0) {
    return FUNC_REPLACEAT;
  } else if(strcmp(name, "insertAt") == 0) {
    return FUNC_INSERTAT;
  } else if(strcmp(name, "squeeze") == 0) {
    return FUNC_SQUEEZE;
  } else if(strcmp(name, "trim") == 0) {
    return FUNC_TRIM;
  } else if(strcmp(name, "compare") == 0) {
    return FUNC_COMPARE;
  } else if(strcmp(name, "toString") == 0) {
    return FUNC_TOSTRING;
  } else if(strcmp(name, "format") == 0) {
    return FUNC_FORMAT;
  }

  return -1;
}

/**
 * Invokes the function with the given function ID with the
 * specified parameters.
 * @param id the function ID to invoke.
 * @param parameters the parameters to invoke the function with.
 * @throws WMLScriptWrongArityException
 * if the number of parameters does not match the expected
 * number of parameters.
 * @throws WMLScriptUnknownLibraryFunctionException
 * if the given function ID does not refer to a known function.
 * @return the return value of the library function.
 */
WMLScriptValueObject *WMLScriptLibraryString::call(int id, WMLScriptFunctionParameters *parameters) {
  switch(id) {
  case FUNC_LENGTH:
    return funcLength(parameters);
  case FUNC_ISEMPTY:
    return funcIsEmpty(parameters);
  case FUNC_CHARAT:
    return funcCharAt(parameters);
  case FUNC_SUBSTRING:
    return funcSubString(parameters);
  case FUNC_FIND:
    return funcFind(parameters);
  case FUNC_REPLACE:
    return funcReplace(parameters);
  case FUNC_ELEMENTS:
    return funcElements(parameters);
  case FUNC_ELEMENTAT:
    return funcElementAt(parameters);
  case FUNC_REMOVEAT:
    return funcRemoveAt(parameters);
  case FUNC_REPLACEAT:
    return funcReplaceAt(parameters);
  case FUNC_INSERTAT:
    return funcInsertAt(parameters);
  case FUNC_SQUEEZE:
    return funcSqueeze(parameters);
  case FUNC_TRIM:
    return funcTrim(parameters);
  case FUNC_COMPARE:
    return funcCompare(parameters);
  case FUNC_TOSTRING:
    return funcToString(parameters);
  case FUNC_FORMAT:
    return funcFormat(parameters);
  }
  throw WMLScriptUnknownLibraryFunctionException();
}

/**
 * Returns the name of the library.
 * @return pointer at library name.
 */
const char *WMLScriptLibraryString::getName() const {
  return "String";
}

WMLScriptValueObject *WMLScriptLibraryString::funcLength(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 1, "length");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0);

  p1->convertTo(dataString);

  if(p1->getType() == dataString)
    returnValue->setValue((int) strlen(p1->getStringValue()));

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcIsEmpty(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 1, "isEmpty");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0);

  p1->convertTo(dataString);

  if(p1->getType() == dataString)
    returnValue->setValue(strlen(p1->getStringValue()) ? false : true);

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcCharAt(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 2, "charAt");
  
  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1);

  p1->convertTo(dataString);

  if(p1->getType() == dataString) {

    /* have to use Float.int really */
    p2->convertTo(dataFloat);

    if(p2->getType() == dataFloat) {

      if(p2->getFloatValue() < strlen(p1->getStringValue()) &&
	 p2->getFloatValue() >= 0) {

	p1->getStringValue()[(int) p2->getFloatValue() + 1] = '\0';
	returnValue->setValue(p1->getStringValue());

      } else {

	returnValue->setValue("");

      }

    }
    
  }

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcSubString(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 3, "subString");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1),
    *p3 = parameters->valueAt(2);

  p1->convertTo(dataString);

  if(p1->getType() == dataString) {

    /* have to use Float.int really */
    p3->convertTo(dataFloat);
    
    if(p3->getType() == dataFloat && p3->getFloatValue() == 0) {
      /* length == 0 -> return empty string */
      returnValue->setValue("");
      
    } else {
      
      p2->convertTo(dataFloat);
      
      if(p2->getType() == dataFloat) {

	if(p2->getFloatValue() >= strlen(p1->getStringValue())) {
	  /* startIndex > length of string -> return empty string */
	  returnValue->setValue("");
	  
	} else {

	  if(p2->getFloatValue() < 0)
	    p2->setValue(0.0);

	  /* length + startIndex > length of string -> 
	     length is remaining length of string */
	  if(p2->getFloatValue() + p3->getFloatValue() >=
	     strlen(p1->getStringValue()))
	    p3->setValue((double) (strlen(p1->getStringValue()) -
					  p2->getFloatValue()));
	    
	  char *s = (char *) malloc((int) (p3->getFloatValue() + 1));

	  if(s == NULL)
	    throw WMLScriptOutOfMemoryException();

	  /* copy substring */
	  memcpy(s, &(p1->getStringValue()[(int) p2->getFloatValue()]), 
		 (int) p3->getFloatValue());
	  s[(int) p3->getFloatValue()] = '\0';
	  returnValue->setValue(s);
	  free(s);

	}

      }

    }

  }

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcFind(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 2, "find");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1);

  p1->convertToCommon(p2, dataString, dataString);

  if(p1->getType() == dataString && p2->getType() == dataString) {

    if(strlen(p2->getStringValue()) == 0) {

      /* zero-length substring returns invalid */
      returnValue->setValue();

    } else {

      char *s = strstr(p1->getStringValue(), p2->getStringValue());
      
      if(s == NULL) {
	
	/* not found */
	returnValue->setValue(-1);
	
      } else {
	
	/* return the offset to the first occurrence of substring */
	returnValue->setValue((int) (s - p1->getStringValue()));
	
      }

    }

  }

  return returnValue;
  
}

WMLScriptValueObject *WMLScriptLibraryString::funcReplace(WMLScriptFunctionParameters *parameters) {
  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1),
    *p3 = parameters->valueAt(2);

  p1->convertTo(dataString);
  p2->convertTo(dataString);
  p3->convertTo(dataString);

  if(p1->getType() == dataString && p2->getType() == dataString &&
     p3->getType() == dataString) {

    /* zero-length oldsubstring returns invalid */
    if(strlen(p2->getStringValue()) != 0) {

      char *s1 = p1->getStringValue();
      char *old = p2->getStringValue();
      char *sub = p3->getStringValue();
      char *p = s1, *prev_p;
      int count = 0, len = strlen(s1), 
	old_len = strlen(old),
	sub_len = strlen(sub), 
	diff = sub_len - old_len;

      /* find number of occurrences of old substring */
      while(p) {
	p = strstr(p, old);
	if(p) {
	  count++;
	  p += old_len;
	}
      }

      char *s2 = (char *) malloc(len + (count * diff) + 1);
      char *p2 = s2;

      if(s2 == NULL)
	throw WMLScriptOutOfMemoryException();

      p = s1;
      prev_p = s1;

      while(p) {
	p = strstr(p, old);

	if(p == NULL) {

	  /* copy remainder */
	  strcpy(p2, prev_p);

	} else {

	  /* copy until first occurrence and copy substitution */
	  memcpy(p2, prev_p, (int) (p - prev_p));
	  strcpy(p2 + (p - prev_p), sub);
	  p2 += (p - prev_p) + (sub_len);
	  p += old_len;
	  prev_p = p;

	}

      }

      returnValue->setValue(s2);
      free(s2);

    }

  }

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcElements(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 2, "elements");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1);

  p1->convertTo(dataString);
  p2->convertTo(dataString);
  
  if(p1->getType() == dataString && p2->getType() == dataString) {

    if(strlen(p2->getStringValue()) != 0) {
      /* separator must be non-empty string */
      char *s1 = p1->getStringValue(), *p = s1;
      char *s2 = p2->getStringValue();
      
      int count = 1, sep = s2[0];
      
      while(p) {
	
	p = strchr(p, sep);
	if(p) {
	  count++;
	  p++;
	}

      }

      returnValue->setValue(count);

    }

  }

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcElementAt(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 3, "elementAt");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1),
    *p3 = parameters->valueAt(2);

  p1->convertTo(dataString);
  p2->convertTo(dataFloat);
  p3->convertTo(dataString);
  
  if(p1->getType() == dataString && p2->getType() == dataFloat &&
     p3->getType() == dataString) {

    if(strlen(p1->getStringValue()) == 0) {
      /* if string is an empty string then an empty string is returned */
      
      returnValue->setValue("");

    } else {


      if(strlen(p3->getStringValue()) != 0) {
	/* separator must be non-empty string */
	
	int count = 0, index = (int) p2->getFloatValue();
	char *p = p1->getStringValue();
	char *sep = p3->getStringValue();
	sep[1] = '\0';
	
	/* if index < 0 then return first element */
	if(index < 0) index = 0;

	p = strtok(p, sep);

	if(p) {

	  while(p) {
	  
	    if(index == count)
	      break;
	    
	    count++;
	    p = strtok(NULL, sep);
	    
	  }

	  if(p == NULL) {
	    /* if the index does not exist return the empty string */
	    p = p1->getStringValue();
	    p[0] = '\0';

	  }

	} else {
	  /* if there are no elements return the empty string */
	  p[0] = '\0';

	}
	
	returnValue->setValue(p);
	
      }

    }

  }

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcRemoveAt(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 3, "removeAt");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1),
    *p3 = parameters->valueAt(2);

  p1->convertTo(dataString);
  p2->convertTo(dataFloat);
  p3->convertTo(dataString);
  
  if(p1->getType() == dataString && p2->getType() == dataFloat &&
     p3->getType() == dataString) {

    if(strlen(p1->getStringValue()) == 0) {
      /* if string is an empty string then an empty string is returned */
      
      returnValue->setValue("");

    } else {

      if(strlen(p3->getStringValue()) != 0) {
	/* separator must be non-empty string */

	char *s = p1->getStringValue(), *p = s, *prev_p = p;
	int sep = p3->getStringValue()[0];
	int elements = 0;

	/* count number of elements */
	while(p) {

	  p = strchr(p, sep);
	  
	  if(p) {
	    elements++;
	    p++;
	  }

	}

	p = prev_p;

	int idx = (int) p2->getFloatValue(), count = 0;
	int start_at = 0;
	int end_at = 0;

	/* clamp index */
	if(idx < 0) idx = 0;
	if(idx > elements) idx = elements;

	/* determine starting and ending positions of element */
	while(p) {
	  
	  p = strchr(p, sep);

	  if(p) {
	    /* got separator */

	    p++;

	    if(idx == count) {
	      
	      start_at = prev_p - s;
	      end_at = p - s;
	      
	    }
	    
	    prev_p = p;
	    count++;

	  } else {
	    /* no more separators -> last element */

	    if(idx == count) {
	      /* index points to last element, so element extends to 
		 remainder of string */

	      start_at = prev_p - s - 1;
	      end_at = strlen(s);

	    }

	  }

	}

	char *s2 = (char *) malloc(strlen(s) - (end_at - start_at) + 1);

	if(s2 == NULL)
	  throw WMLScriptOutOfMemoryException();

	/* copy part preceding element and part following element */
	memcpy(s2, s, start_at);
	memcpy(&s2[start_at], &s[end_at], strlen(s) - end_at);
	s2[strlen(s) - (end_at - start_at)] = '\0';

	returnValue->setValue(s2);
	free(s2);
	
      }

    }

  }

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcReplaceAt(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 4, "replaceAt");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1),
    *p3 = parameters->valueAt(2),
    *p4 = parameters->valueAt(3);

  p1->convertTo(dataString);
  p2->convertTo(dataString);
  p3->convertTo(dataFloat);
  p4->convertTo(dataString);
  
  if(p1->getType() == dataString && p2->getType() == dataString &&
     p3->getType() == dataFloat && p4->getType() == dataString) {

    if(strlen(p1->getStringValue()) == 0) {
      /* if string is an empty string then the given element is returned */
      
      returnValue->setValue(p2->getStringValue());

    } else {

      if(strlen(p4->getStringValue()) != 0) {
	/* separator must be non-empty string */

	char *s = p1->getStringValue(), *p = s, *prev_p = p;
	int sep = p4->getStringValue()[0];
	int elements = 0;

	/* count number of elements */
	while(p) {

	  p = strchr(p, sep);
	  
	  if(p) {
	    elements++;
	    p++;
	  }

	}

	p = prev_p;

	int idx = (int) p3->getFloatValue(), count = 0;
	int start_at = 0;
	int end_at = 0;

	/* clamp index */
	if(idx < 0) idx = 0;
	if(idx > elements) idx = elements;

	/* determine starting and ending positions of element */
	while(p) {
	  
	  p = strchr(p, sep);

	  if(p) {
	    /* got separator */

	    p++;

	    if(idx == count) {
	      
	      start_at = prev_p - s;
	      end_at = p - s - 1;
	      
	    }
	    
	    prev_p = p;
	    count++;

	  } else {
	    /* no more separators -> last element */

	    if(idx == count) {
	      /* index points to last element, so element extends to 
		 remainder of string */

	      start_at = prev_p - s - 1 + 1;
	      end_at = strlen(s);

	    }

	  }

	}

	char *s2 = (char *) malloc(strlen(s) - (end_at - start_at) + 
				   strlen(p2->getStringValue()) + 1);

	if(s2 == NULL)
	  throw WMLScriptOutOfMemoryException();

	/* copy part preceding old element, insert new element,
	   then append part following old element */
	memcpy(s2, s, start_at);
	memcpy(&s2[start_at], p2->getStringValue(), 
	       strlen(p2->getStringValue()));
	memcpy(&s2[start_at + strlen(p2->getStringValue())], 
		  &s[end_at], strlen(s) - end_at);
	s2[strlen(p2->getStringValue()) + 
	  strlen(s) - (end_at - start_at)] = '\0';

	returnValue->setValue(s2);
	free(s2);
	
      }

    }

  }

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcInsertAt(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 4, "insertAt");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1),
    *p3 = parameters->valueAt(2),
    *p4 = parameters->valueAt(3);

  p1->convertTo(dataString);
  p2->convertTo(dataString);
  p3->convertTo(dataFloat);
  p4->convertTo(dataString);
  
  if(p1->getType() == dataString && p2->getType() == dataString &&
     p3->getType() == dataFloat && p4->getType() == dataString) {

    if(strlen(p1->getStringValue()) == 0) {
      /* if string is an empty string then the given element is returned */
      
      returnValue->setValue(p2->getStringValue());

    } else {

      if(strlen(p4->getStringValue()) != 0) {
	/* separator must be non-empty string */

	char *s = p1->getStringValue(), *p = s, *prev_p = p;
	int sep = p4->getStringValue()[0];
	int elements = 0;

	/* count number of elements */
	while(p) {

	  p = strchr(p, sep);
	  
	  if(p) {
	    elements++;
	    p++;
	  }

	}

	p = prev_p;

	int idx = (int) p3->getFloatValue(), count = 0;
	int start_at = 0;

	/* clamp index */
	if(idx < 0) idx = 0;
	if(idx > elements) idx = elements + 1;

	/* determine starting and ending positions of element */
	while(p) {
	  
	  p = strchr(p, sep);

	  if(p) {
	    /* got separator */

	    p++;

	    if(idx == count) {
	      
	      start_at = prev_p - s;
	      break;

	    }
	    
	    prev_p = p;
	    count++;

	  } else {
	    /* no more separators -> last element */

	    if(idx == elements + 1) {
	      /* index points to last element, so element extends to 
		 remainder of string */

	      start_at = strlen(s);

	    } 
	    
	    if(count == idx) {

	      start_at = prev_p - s;

	    }
	    
	  }

	}

	char *s2 = (char *) malloc(strlen(s) + 
				   strlen(p2->getStringValue()) + 3);

	if(s2 == NULL)
	  throw WMLScriptOutOfMemoryException();

	p = s2;

	/* copy part preceding old element, insert new element,
	   then append part following old element */
	memcpy(p, s, start_at);
	p += start_at;

	if(start_at && p[-1] != (char) sep) {

	  *p = (char) sep;
	  p++;

	}

	memcpy(p, p2->getStringValue(), 
	       strlen(p2->getStringValue()));
	p += strlen(p2->getStringValue());

	if(start_at != (int) strlen(s) && p[1] != (char) sep) {

	  *p = (char) sep;
	  p++;

	}

	memcpy(p, &s[start_at], strlen(s) - start_at);
	p += strlen(s) - start_at;

	*p = '\0';

	returnValue->setValue(s2);
	free(s2);
	
      }

    }

  }

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcSqueeze(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 1, "squeeze");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0);

  p1->convertTo(dataString);

  if(p1->getType() == dataString) {

    int i = 0, j = 0, len = strlen(p1->getStringValue());
    bool skipping = false;
    char *s1 = (char *) p1->getStringValue();
    char *s2 = (char *) malloc(len + 1);

    s2[0] = '\0';

    for(i = 0; i < len; i++) {
      
      if(s1[i] != '\r' && s1[i] != '\n' && s1[i] != ' ' && s1[i] != '\t' &&
	 s1[i] != '\f' && s1[i] != '\f') {

	/* copy non-whitespace character */
	skipping = false;
	s2[j++] = s1[i];
	
      } else {

	/* if this is not the first whitespace character, skip it */
	if(skipping)
	  continue;

	/* if this is the first whitespace character, copy a 
	   single space */
	skipping = true;
	s2[j++] = ' ';

      }

    }

    s2[j] = '\0';

    returnValue->setValue(s2);
    free(s2);

  }
  
  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcTrim(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 1, "trim");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0);

  p1->convertTo(dataString);

  if(p1->getType() == dataString) {

    int i = 0, j = 0, begin = 0, end = 0, len = strlen(p1->getStringValue());
    char *s1 = (char *) p1->getStringValue();
    char *s2 = (char *) malloc(len + 1);

    s2[0] = '\0';

    /* skip whitespace at beginning of string */
    for(i = 0; i < len; i++) {
      
      if(s1[i] != '\r' && s1[i] != '\n' && s1[i] != ' ' && s1[i] != '\t' &&
	 s1[i] != '\f' && s1[i] != '\f') {

	begin = i;
	break;

      }

    }

    /* skip whitespace at end of string */
    for(i = len - 1; i >= 0; i--) {

      if(s1[i] != '\r' && s1[i] != '\n' && s1[i] != ' ' && s1[i] != '\t' &&
	 s1[i] != '\f' && s1[i] != '\f') {

	end = i + 1;
	break;
	
      }

    }

    /* copy non-whitespace */
    for(i = begin; i < end; i++)
	s2[j++] = s1[i];

    s2[j] = '\0';

    returnValue->setValue(s2);
    free(s2);

  }
  
  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcCompare(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 2, "compare");
  
  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1);

  p1->convertToCommon(p2, dataString, dataString);

  if(p1->getType() == dataString && p2->getType() == dataString) {

    int c = strcmp(p1->getStringValue(), p2->getStringValue());
  
    if(c > 0) c = 1;
    if(c < 0) c = -1;
    returnValue->setValue(c);

  }

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcToString(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 1, "toString");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0);

  char *s = p1->toString();
  returnValue->setValue(s);
  free(s);

  return returnValue;
}

WMLScriptValueObject *WMLScriptLibraryString::funcFormat(WMLScriptFunctionParameters *parameters) {
  paramCheck(parameters, 2, "format");

  WMLScriptValueObject *returnValue = new WMLScriptValueObject(),
    *p1 = parameters->valueAt(0),
    *p2 = parameters->valueAt(1);

  p1->convertTo(dataString);

  if(p1->getType() == dataString) {

    char *s = p1->getStringValue(), *p = s, *prev_p = p;
    int width = -1, precision = -1, spec_start = 0, spec_len = 0;
    bool formatValid = true;
    char type, rtype;

    p = strchr(p, '%');

    if(p) {
      /* got first format specifier */
      spec_start = (p - s);
      p++;

      p = parseFormat(p, &type, &width, &precision);

      if(p) {
	/* first format specifier is valid */
	spec_len = (int) ((p - s) - spec_start);

	/* replace remaining format specifiers by empty string */
	p = strchr(p, '%');

	while(p) {
	  p++;
	  prev_p = p;
	  p = parseFormat(p, &rtype, NULL, NULL);

	  if(p == NULL) {
	    /* invalid format specifier returns invalid */

	    formatValid = false;
	    break;

	  }

	  if(rtype == '%') 
	    p--;

	  memmove(prev_p - 1, p, strlen(p) + 1);
	  
	  p = strchr(prev_p, '%');

	}

      } else {
	/* invalid format specifier returns invalid */

	formatValid = false;

      }

    }

    if(formatValid) {
      /* format is valid and guaranteed to contain at most 1 
	 specifier */
      p = strchr(s, '%');
      
      if(p) {
	/* format contains specifier -> perform substitution */
	s = substFormat(s, spec_start, spec_len, type, width, precision, p2);

	if(s) {
	  /* substitution was valid -> return substituted format string */
	  returnValue->setValue(s);
	  free(s);

	}

      } else {
	/* format does not contain specifier -> return format string */
	returnValue->setValue(s);

      }

    }

  }

  return returnValue;

}


char *WMLScriptLibraryString::substFormat(char *s, int offset, int len, char type, int width, int precision, WMLScriptValueObject *val) {
  char *s2 = NULL;

  switch(type) {
  case 'd':
    val->convertTo(dataFloat);

    if(val->getType() == dataFloat)
      s2 = intFormat(s, offset, len, width, precision, val);
    break;
  case 'f':
    val->convertTo(dataFloat);

    if(val->getType() == dataFloat)
      s2 = floatFormat(s, offset, len, width, precision, val);
    break;
  case 's':
    val->convertTo(dataString);

    if(val->getType() == dataString)
      s2 = stringFormat(s, offset, len, width, precision, val);
    break;
  case '%':
    s2 = (char *) malloc(strlen(s));
    strcpy(s2, s);
    memmove(&s2[offset], &s2[offset + 1], strlen(s) - (offset + len));
    s2[strlen(s2) - 1] = '\0';
    break;
  default:
    printf("substFormat: invalid format");
  }

  return s2;
}

char *WMLScriptLibraryString::intFormat(char *s, int offset, int len, int width, int precision, WMLScriptValueObject *val) {
  if(precision == -1) precision = 1;

  val->setValue((int) val->getFloatValue());
  char *s2 = (char *) malloc(strlen(s) + (8 * sizeof(val->getIntValue()))), *p2 = s2;
  char *out = (char *) malloc(8 * sizeof(val->getIntValue()));

  sprintf(out, "%d", abs(val->getIntValue()));
  
  int i = 0;

  /* left fill with zeroes up to precision */
  if((int) strlen(out) < precision) {

    int w = strlen(out);
    memmove(&out[precision - w], out, w + 1);

    for(i = 0; i < precision - w; i++)
      out[i] = '0';

  }

  if(val->getIntValue() < 0) {
    
    memmove(&out[1], out, strlen(out) + 1);
    out[0] = '-';
    
  }

  /* if precision is 0 and value is 0 -> return empty string */
  if(precision == 0 && val->getIntValue() == 0) 
    out[0] = '\0';

  /* left fill with spaces up to width */
  if((int) strlen(out) < width) {

    int w = strlen(out);
    memmove(&out[width - w], out, w + 1);
    for(i = 0; i < width - w; i++)
      out[i] = ' ';

  }

  p2 = s2 + offset;

  for(i = 0; i < (int) strlen(out); i++)
    *(p2++) = out[i];

  memcpy(s2, s, offset);  
  strcpy(p2, &s[offset + len]);

  free(out);

  return s2;
}

char *WMLScriptLibraryString::floatFormat(char *s, int offset, int len, int width, int precision, WMLScriptValueObject *val) {

  if(width == -1)
    width = 1;
  if(precision == -1)
    precision = 6;

  char *s2 = (char *) malloc(250 + strlen(s));
  char *out = (char *) malloc(250);
  char *fmt = (char *) malloc(25);
  
  sprintf(fmt, "%%%d.%df", width, precision);
  sprintf(out, fmt, val->getFloatValue());
  
  free(fmt);

  memcpy(s2, s, offset);  
  strcpy(&s2[offset], out);
  strcpy(&s2[offset + strlen(out)], &s[offset + len]);

  free(out);

  return s2;
}


char *WMLScriptLibraryString::stringFormat(char *s, int offset, int len, int width, int precision, WMLScriptValueObject *val) {
  if(width == -1) 
    width = strlen(val->getStringValue());
  if(precision == -1) 
    precision = strlen(val->getStringValue());
  char *s2 = (char *) malloc(strlen(s) + strlen(val->getStringValue()) + 1);
  char *out = (char *) malloc(250);
  char *fmt = (char *) malloc(25);
  
  sprintf(fmt, "%%%d.%ds", width, precision);
  sprintf(out, fmt, val->getStringValue());
  
  free(fmt);

  memcpy(s2, s, offset);  
  strcpy(&s2[offset], out);
  strcpy(&s2[offset + strlen(out)], &s[offset + len]);

  free(out);  

  return s2;
}

char *WMLScriptLibraryString::parseFormat(char *p, char *type, int *width, int *precision) {
  if(p[0]) {
    /* need another character */
	
    if(isdigit(p[0])) {
      /* width argument */

      int i = 0, j = 0;

      while(isdigit(p[i++]));

      if(width) {

	char *sw = (char *) malloc(i);
	
	if(sw == NULL)
	  throw WMLScriptOutOfMemoryException();
	
	for(j=0; j < i - 1; j++)
	  sw[j] = p[j];
	sw[j] = '\0';
	
	*width = atoi(sw);

	free(sw);

      }

      p += (i - 1);

    }

    if(p[0] == '.') {
      /* precision argument */
	  
      p++;
      int i = 0, j = 0;
      while(isdigit(p[i++]));

      if(precision) {

	char *sp = (char *) malloc(i);
	
	if(sp == NULL)
	  throw WMLScriptOutOfMemoryException();
	
	for(j = 0; j < i - 1; j++) 
	  sp[j] = p[j];
	sp[j] = '\0';

	*precision = atoi(sp);

	free(sp);

      }

      p += (i - 1);

    }

    /* type */
    switch(p[0]) {
    case 'd':
    case 'f':
    case 's':
    case '%':
      if(type)
	*type = p[0];
      return &p[1];
      break;
    default:
      return NULL;
    }

  }

  return NULL;

}
