/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptLibraryURL.h,v 1.5 2001/01/27 14:36:41 wouter Exp $
 */

#ifndef __WMLSCRIPTLIBRARYURL_H__
#define __WMLSCRIPTLIBRARYURL_H__

#include "WMLScriptLibrary.h"
#include "../lib/URI.h"

#define FUNC_ISVALID 0
#define FUNC_GETSCHEME 1
#define FUNC_GETHOST 2
#define FUNC_GETPORT 3
#define FUNC_GETPATH 4
#define FUNC_GETPARAMETERS 5
#define FUNC_GETQUERY 6
#define FUNC_GETFRAGMENT 7
#define FUNC_GETBASE 8
#define FUNC_GETREFERER 9
#define FUNC_RESOLVE 10
#define FUNC_ESCAPESTRING 11
#define FUNC_UNESCAPESTRING 12
#define FUNC_LOADSTRING 13


/**
 * This class implements the URL library as specified
 * by WAP-194-WMLScript Standard Libraries Specification [June 2000].
 * The setBaseURL() method must be used to ensure that
 * URL.getBase() will return meaningful results.
 */
class WMLScriptLibraryURL : public WMLScriptLibrary {
 public:
  WMLScriptLibraryURL();
  ~WMLScriptLibraryURL();

  int getFunctionID(const char *name) const;
  WMLScriptValueObject *call(int id, WMLScriptFunctionParameters *parameters);
  const char *getName() const;

  const URI *getBaseURI() const;
  const URI *getRefererURI() const;

  void setBaseURI(URI *uri);
  void setRefererURI(URI *url);

 private:
  /**
   * Stores URL set with setBaseURL().
   */
  URI *baseURI;
  /**
   * Stores URL set with setRefererURL().
   */
  URI *refererURI;

  WMLScriptValueObject *funcIsValid(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcGetScheme(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcGetHost(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcGetPort(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcGetPath(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcGetParameters(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcGetQuery(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcGetFragment(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcGetBase(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcGetReferer(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcResolve(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcEscapeString(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcUnescapeString(WMLScriptFunctionParameters *parameters);
  WMLScriptValueObject *funcLoadString(WMLScriptFunctionParameters *parameters);

};

#endif


