/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptParseResult.h,v 1.8 2001/01/27 14:36:42 wouter Exp $
 */

#ifndef __WMLSCRIPTPARSERESULT_H__
#define __WMLSCRIPTPARSERESULT_H__

#include "../lib/Garbage.h"

#include "WMLScriptPragma.h"

#define ERRORMSG_SIZE 255
/**
 * This structure allows the C++ WMLScriptParser object to interact with
 * the C-based parser and lexer in a reentrant-friendly manner. The lexer
 * is still not completely reentrant however, thus rendering the entire
 * parser non-reentrant.
 */
typedef struct WMLScriptParseResultTag {
  /** number of pointers in the functionTable. */
  int numberOfFunctions;
  /** pointer to an array of WMLScriptFunctions. */
  WMLScriptFunction *functionTable;
  /** the number of pragma's. */
  int numberOfPragmas;
  /** pointer to an array of pragma pointers (different types of 
      pragma's occupy different amounts of memory). */
  WMLScriptPragma **pragmaTable;
  /** a table that tracks pointers which have to be released
      if a parse error occurs. */
  GarbageTable conditionalGarbage;
  /** a table that tracks pointers which have to be released
      whether the parse is successfull or not. */
  GarbageTable unconditionalGarbage;
  /** line number where parse stopped. */
  int lineNumber;
  /** column where parse stopped. */
  int columnNumber;
  /** parser/lexer error message. */
  char parseErrorMessage[ERRORMSG_SIZE];
} WMLScriptParseResult;

#endif



