/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptPragma.h,v 1.5 2001/01/27 14:36:42 wouter Exp $
 */

#ifndef __WMLSCRIPTPRAGMA_H__
#define __WMLSCRIPTPRAGMA_H__

/**
 * Enumerates the different kinds of pragma's.
 */
typedef enum {
  typeExternalCompilationUnit,
  typeAccess,
  typeMeta
} WMLScriptPragmaType;

/**
 * Enumerates the different kinds of meta pragma's.
 */
typedef enum {
  typeName,
  typeHTTPEquiv,
  typeUserAgent
} WMLScriptMetaPragmaType;

/**
 * Describes an external compilation unit pragma.
 */
typedef struct {
  WMLScriptPragmaType type;
  /**
   * Identifier to refer to the external compilation unit.
   */
  char *id;
  /**
   * URL of external compilation unit.
   */
  char *url;
} WMLScriptExternalCompilationUnitPragma;

/**
 * Describes an access pragma.
 */
typedef struct {
  WMLScriptPragmaType type;
  /**
   * Domain to use for suffix matching. May be NULL if path is non-NULL.
   */
  char *domain;
  /**
   * Path to use for suffix matching. May be NULL if domain is non-NULL.
   */
  char *path;
} WMLScriptAccessPragma;

/**
 * Describes a meta pragma.
 */
typedef struct {
  WMLScriptPragmaType type;
  /**
   * Contains the kind of meta pragma.
   */
  WMLScriptMetaPragmaType metaType;
  /**
   * The key for this meta pragma.
   */
  char *key;
  /**
   * The value for this meta pragma.
   */
  char *value;
  /**
   * The scheme for this meta pragma (may be NULL).
   */
  char *scheme;
} WMLScriptMetaPragma;

/**
 * Describes any pragma.
 */
typedef union {
  WMLScriptPragmaType type;
  /**
   * If type is typeExternalCompilationUnit, then this field is meaningful.
   */
  WMLScriptExternalCompilationUnitPragma ext;
  /**
   * If type is typeAccess, then this field is meaningful.
   */
  WMLScriptAccessPragma access;
  /**
   * If type is typeMeta, then this field is meaningful.
   */
  WMLScriptMetaPragma meta;
} WMLScriptPragma;

#endif
