/* 
 * Copyright (c) 2000-2001, catchy.net. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *      - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 * 
 *      - Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 * 
 *      - All advertising materials mentioning features or use of this software
 *      must display the following acknowledgement: This product includes
 *      software developed by catchy.net and its contributors.
 * 
 *      - Neither name of catchy.net nor the names of its contributors may
 *      be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY CATCHY.NET AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL CATCHY.NET OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * $Id: WMLScriptReturnException.cpp,v 1.4 2001/01/27 14:36:42 wouter Exp $
 */

#include "WMLScriptValueObject.h"
#include "WMLScriptReturnException.h"

/**
 * Create a return exception at a given line and column number.
 * with an undefined message.
 * @return the return value.
 */
WMLScriptReturnException::WMLScriptReturnException(WMLScriptValueObject *val, int l, int c) : WMLScriptRuntimeException(l, c) {
  value = val;
};

/**
 * Create a return exception at an unknown line and column number.
 * with a specified message.
 * @return the return value.
 */
WMLScriptReturnException::WMLScriptReturnException(WMLScriptValueObject *val, char *message) : WMLScriptRuntimeException(message) {
  value = val;
};

/**
 * Create a return exception at a given line and column number.
 * with a specified message.
 * @return the return value.
 */
WMLScriptReturnException::WMLScriptReturnException(WMLScriptValueObject *val, char *message, int l, int c) : WMLScriptRuntimeException(message, l, c) {
  value = val;
};

/**
 * Destroy the return exception.
 */
WMLScriptReturnException::~WMLScriptReturnException() {
}

/**
 * Get the return value associated with this exception.
 * @return return value.
 */
WMLScriptValueObject *WMLScriptReturnException::getValue() {
  return value;
}
